import sys

import lib.common.os
from config import gce_dir_path
from lib.xuyuxing.base.common_command import millify
import lib.xuyuxing.base.base_function as bf

def GenomeSurvey_main(args):
    kmer_freq_hash_path = gce_dir_path + "kmerfreq/kmer_freq_hash/kmer_freq_hash"
    gce_path = gce_dir_path + "gce"

    genome_size = args.genome_size
    genome_coverage = args.genome_coverage
    input_file_list = args.input_file_list
    kmer = args.kmer
    min_accuracy_rate = args.min_accuracy_rate
    quality_value = args.quality_value
    thread_number = args.thread_number
    output_prefix = args.output_prefix
    output_kmer_frequence_file = args.output_kmer_frequence_file
    print_flag = args.print_info
    maximum_read_length = args.maximum_read_length
    begin_ignore_len = args.begin_ignore_len
    end_ignore_len = args.end_ignore_len

    sequencing_error_rate = 1 - min_accuracy_rate
    F_represent = 0.75

    max_number_of_error_kmer = int(
        genome_coverage * kmer * sequencing_error_rate * genome_size)
    total_kmer_species_number = int(genome_size + max_number_of_error_kmer)
    initial_size_of_hash_table = int(
        total_kmer_species_number / thread_number / F_represent)

    peak_size_of_running_memory = int(
        initial_size_of_hash_table * thread_number * 8)

    cmd_string = "%s -a %d -d %d -k %d -L %d -l %s -c %.2f -q %d -i %d -t %d -p %s -o %d" % (
        kmer_freq_hash_path, begin_ignore_len, end_ignore_len, kmer, maximum_read_length, input_file_list, min_accuracy_rate, quality_value, initial_size_of_hash_table,
        thread_number, output_prefix, output_kmer_frequence_file)

    if print_flag is True:
        print("""
        Estimated genome size:          %s
        Coverage of sequence data:      %dX
        k-mer:                          %d
        min accuracy rate:              %f
        thread number:                  %d
        F represent:                    %f
        max number of error kmer:       %d
        total kmer species number:      %d
        initial size of hash table:     %d
        peak size of running memory:    %s
        """ % (
            millify(
                genome_size), genome_coverage, kmer, min_accuracy_rate, thread_number, F_represent,
            max_number_of_error_kmer,
            total_kmer_species_number, initial_size_of_hash_table, millify(peak_size_of_running_memory)))
        print(cmd_string)
        sys.exit(0)

    print(cmd_string)

    flag, output, error = lib.common.os.cmd_run(cmd_string)

    print(error)

    # kmer_report_string = error.decode()
    #
    #
    # for each_line in kmer_report_string.split("\n"):
    #    matchobj = re.match(r"^(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+\.\d+)\s+(\d+)\s+(\d+)\s+(\d+\.\d+)\s+(\d+)\s+(\d+)")
    #    if matchobj:
    #        Kmer_individual_num = matchobj.group(2)


import matplotlib.pyplot as plt
from matplotlib.path import Path
import matplotlib.patches as Patches
import matplotlib.ticker as ticker
import matplotlib.patheffects as path_effects
from matplotlib.collections import PatchCollection
import re
from lib.common.genome.genome_feature2 import read_gff_file, convert_dict_structure
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.fileIO import read_list_file, tsv_file_dict_parse
from api.xuyuxing.plot.base import add_stick_box_plot, add_small_axes_on_other
from collections import OrderedDict


class chromesome(object):
    def __init__(self, chr_id, gf_list, fasta_record):
        self.id = chr_id
        self.length = fasta_record.len()
        self.gf_list = sorted(gf_list, key=lambda x: x.start)

    def __str__(self):
        return '%s (%d bp): %d genes on forward strand, %d genes on reverse strand' % (self.id, self.length, len([i for i in self.gf_list if i.strand == '+']), len([i for i in self.gf_list if i.strand == '-']))

    __repr__ = __str__

    def gene_site_list(self):
        site_list = []
        for gf in self.gf_list:
            site_list.append((gf.id, gf.start, gf.end, gf.strand))
        return site_list


def build_chr_dict(gff_file, fasta_file):
    gff_dict = read_gff_file(gff_file)
    gene_dict, gf_chr_dict = convert_dict_structure(gff_dict)
    fasta_record_dict = read_fasta_by_faidx(fasta_file)

    chr_dict = {}
    for chr_id in fasta_record_dict:
        if chr_id in gf_chr_dict:
            gf_list = [gf_chr_dict[chr_id]['+'][i] for i in gf_chr_dict[chr_id]['+']
                       ] + [gf_chr_dict[chr_id]['-'][i] for i in gf_chr_dict[chr_id]['-']]
        else:
            gf_list = []
        chr_dict[chr_id] = chromesome(
            chr_id, gf_list, fasta_record_dict[chr_id])

    return chr_dict


def add_chr_patch(ax, start, end, high=5, x_y_lim=None, thickness=2, linewidth=2, color='b', chr_id=None, chr_id_site='down'):
    # add outline
    outline_patch = add_stick_box_plot(start, end, high=high, x_y_lim=x_y_lim, thickness=thickness,
                                       linewidth=linewidth, alpha=1, facecolor='none', edgecolor='k', bulge=3)
    # add inner box
    inner_patch = add_stick_box_plot(start, end, high=high, x_y_lim=x_y_lim, thickness=thickness *
                                     0.6, linewidth=0, alpha=0.5, facecolor=color, edgecolor='none', bulge=2)

    # add chr name
    if chr_id:
        if chr_id_site == 'down':
            ax.text(start, high-0.5, chr_id)
        else:
            ax.text(start, high+0.2, chr_id)

    # add chr tick

    ax.add_patch(outline_patch)
    ax.add_patch(inner_patch)


def one_way_transformation(input_data, range_input, range_output):
    return ((input_data - range_input[0]) / (range_input[1]-range_input[0]) * (range_output[1]-range_output[0])) + range_output[0]


def plot_genome(ax, chr_dict, plot_chr_id_list, plot_size_range, high, x_y_lim=None, thickness=0.2, linewidth=2, color='b'):
    plot_chr_list = [chr_dict[i] for i in plot_chr_id_list]
    plot_genome_len = sum([i.length for i in plot_chr_list])
    gap_sum_length = plot_genome_len * 0.1
    gap_num = len(plot_chr_id_list) - 1
    gap_length = gap_sum_length/gap_num
    plot_num_range = (0, plot_genome_len + gap_sum_length)

    start_point = 0
    chr_start_dict = {}
    for chr_tmp in plot_chr_list:
        add_chr_patch(ax, one_way_transformation(start_point, plot_num_range, plot_size_range), one_way_transformation(
            start_point+chr_tmp.length, plot_num_range, plot_size_range), high=high, x_y_lim=x_y_lim, thickness=thickness, linewidth=linewidth, color=color)
        chr_start_dict[chr_tmp.id] = start_point
        # print(start_point, start_point+chr_tmp.length)
        start_point = start_point+chr_tmp.length + gap_length

    return chr_start_dict, plot_num_range

# two genome synteny compare plot


def add_map_bar(query_range, subject_range, q_h, s_h, curve_deep=1):

    q_s, q_e = query_range
    s_s, s_e = subject_range

    path_data = [
        # q_s curve4
        (Path.MOVETO, [q_s, q_h]),
        (Path.CURVE4, [q_s, q_h - curve_deep]),
        (Path.CURVE4, [s_s, s_h + curve_deep]),
        (Path.CURVE4, [s_s, s_h]),
        # s_line
        (Path.LINETO, [s_e, s_h]),
        # q_e curve4
        (Path.CURVE4, [s_e, s_h + curve_deep]),
        (Path.CURVE4, [q_e, q_h - curve_deep]),
        (Path.CURVE4, [q_e, q_h]),
        # q_e curve4
        (Path.LINETO, [q_s, q_h]),
    ]

    codes, verts = zip(*path_data)
    path = Path(verts, codes)

    patch = Patches.PathPatch(
        path, alpha=0.2, facecolor='black', edgecolor='black', linewidth=0)

    return patch


def add_map_by_fancy_name(q_chr_id, q_s, q_e, s_chr_id, s_s, s_e, strand, query_chr_start_dict, query_plot_range, subject_chr_start_dict, subject_plot_range, plot_size_range, query_high, subject_high, curve_deep=1):

    q_s = min([q_s, q_e])
    q_e = max([q_s, q_e])

    if strand == '-':
        s_s = max([s_s, s_e])
        s_e = min([s_s, s_e])
    else:
        s_s = min([s_s, s_e])
        s_e = max([s_s, s_e])

    q_s = query_chr_start_dict[q_chr_id]+int(q_s)
    q_e = query_chr_start_dict[q_chr_id]+int(q_e)
    s_s = subject_chr_start_dict[s_chr_id]+int(s_s)
    s_e = subject_chr_start_dict[s_chr_id]+int(s_e)

    q_s = one_way_transformation(q_s, query_plot_range, plot_size_range)
    q_e = one_way_transformation(q_e, query_plot_range, plot_size_range)
    s_s = one_way_transformation(s_s, subject_plot_range, plot_size_range)
    s_e = one_way_transformation(s_e, subject_plot_range, plot_size_range)

    map_bar = add_map_bar((q_s, q_e), (s_s, s_e), query_high,
                          subject_high, curve_deep=curve_deep)

    return map_bar


def two_genome_synteny_block_plot(ax, plot_size_range, x_y_lim, query_id, query_contig_list, query_gff_file, query_fasta_file, subject_id, subject_contig_list, subject_gff_file, subject_fasta_file, query_vs_subject_collinearity_csv_file):

    query_chr_dict = build_chr_dict(query_gff_file, query_fasta_file)
    subject_chr_dict = build_chr_dict(subject_gff_file, subject_fasta_file)

    query_chr_start_dict, query_genome_plot_range = plot_genome(
        ax, query_chr_dict, query_plot_chr_id_list, plot_size_range, 8, x_y_lim=x_y_lim, color='b')
    subject_chr_start_dict, subject_genome_plot_range = plot_genome(
        ax, subject_chr_dict, subject_plot_chr_id_list, plot_size_range, 4, x_y_lim=x_y_lim, color='r')

    tmp_info_dict = tsv_file_dict_parse(query_vs_subject_collinearity_csv_file, fieldnames=[
                                        "query_id", "q_chr", "query_from", "query_to", "subject_id", "s_chr", "subject_from", "subject_to", "strand"])
    col_dict = {}
    for i in tmp_info_dict:
        col_dict[i] = {}
        col_dict[i][tmp_info_dict[i]["query_id"]] = (tmp_info_dict[i]["q_chr"], int(
            tmp_info_dict[i]["query_from"]), int(tmp_info_dict[i]["query_to"]))
        col_dict[i][tmp_info_dict[i]["subject_id"]] = (tmp_info_dict[i]["s_chr"], int(
            tmp_info_dict[i]["subject_from"]), int(tmp_info_dict[i]["subject_to"]))
        col_dict[i]['strand'] = tmp_info_dict[i]["strand"]

    for i in col_dict:
        map_bar = add_map_by_fancy_name(col_dict[i][query_id][0], col_dict[i][query_id][1], col_dict[i][query_id][2], col_dict[i][subject_id][0], col_dict[i][subject_id][1], col_dict[i][subject_id][2], col_dict[i][subject_id], query_chr_start_dict,
                                        query_genome_plot_range, subject_chr_start_dict, subject_genome_plot_range, plot_size_range, 8, 4, curve_deep=1)
        ax.add_patch(map_bar)


# genome cover plot
def divide_into_row(plot_chr_id_list, chr_length_dict, base_per_row):
    """
    plot_chr_id_list = ['Chr1','Chr2']
    chr_length_dict = {'Chr1':10000,'Chr2':20000}
    base_per_row = 50000
    """

    row_plot_dict = {}
    row_num = 1
    for chr_id in plot_chr_id_list:
        if len(row_plot_dict) == 0:
            now_row = row_num
            row_plot_dict[now_row] = []
            row_plot_dict[now_row].append(chr_id)
            base_in_row = chr_length_dict[chr_id]
        elif base_in_row + chr_length_dict[chr_id] > base_per_row:
            now_row = now_row + 1
            row_plot_dict[now_row] = []
            row_plot_dict[now_row].append(chr_id)
            base_in_row = chr_length_dict[chr_id]
        else:
            row_plot_dict[now_row].append(chr_id)
            base_in_row += chr_length_dict[chr_id]

    gap_num = 0
    for i in row_plot_dict:
        gap_num += len(row_plot_dict[i]) - 1

    plot_genome_len = sum([chr_length_dict[i] for i in plot_chr_id_list])
    gap_sum_length = plot_genome_len * 0.1
    gap_length = gap_sum_length/gap_num

    chr_site_dict = {}
    row_length_list = []
    for r_id in row_plot_dict:
        start_tmp = 0
        for chr_id in row_plot_dict[r_id]:
            chr_site_dict[chr_id] = (r_id, start_tmp)
            start_tmp += (chr_length_dict[chr_id] + gap_length)
        row_length_list.append(start_tmp - gap_length)

    max_row_length = max(row_length_list)

    return row_plot_dict, chr_site_dict, gap_length, max_row_length


def plot_genome_by_row(ax, row_plot_dict, chr_site_dict, max_row_length, chr_length_dict, plot_size_range, top_row_high, row_spacing, x_y_lim=None, thickness=0.2, linewidth=2, color='b'):

    plot_num_range = (0, max_row_length)

    row_high = top_row_high

    output_dict = {}

    for r_id in row_plot_dict:
        for chr_id in row_plot_dict[r_id]:
            start_point = chr_site_dict[chr_id][1]
            output_dict[chr_id] = ((one_way_transformation(start_point, plot_num_range, plot_size_range), one_way_transformation(
                start_point+chr_length_dict[chr_id], plot_num_range, plot_size_range)), row_high, thickness)
            add_chr_patch(ax, one_way_transformation(start_point, plot_num_range, plot_size_range), one_way_transformation(
                start_point+chr_length_dict[chr_id], plot_num_range, plot_size_range), high=row_high, x_y_lim=x_y_lim, thickness=thickness, linewidth=linewidth, color=color, chr_id=chr_id)

        row_high = row_high - row_spacing

    return output_dict


def add_cover_plot(ax, chr_cover_dict, chr_length):
    for depth in chr_cover_dict:
        for start, end in chr_cover_dict[depth]:
            patch = add_stick_box_plot(start, end, high=depth, thickness=0.2,
                                       linewidth=0, alpha=0.5, facecolor='g', edgecolor='k', bulge=2)
            ax.add_patch(patch)


def genome_cover_plot(plot_chr_id_list, chr_length_dict, base_cover_chr_dict, base_per_row=200000000, row_spacing=3, fig_size=(20, 15), genome_stick_color='b', genome_stick_thickness=0.2, genome_stick_linewidth=2, max_cover_depth=3):
    """
    plot_chr_id_list = ['Chr1','Chr2']
    chr_length_dict = OrderDict({'Chr1':100000,'Chr2':80000})
    base_cover_chr_dict =  {'Chr1': {1: [(30830, 404093), (1896881, 2312291), (3588453, 3734439)], 2:[(1,10000)]}
    base_per_row = 200000000
    row_spacing = 3
    fig_size = (20,15)
    genome_stick_color = 'b'
    genome_stick_thickness = 0.2
    genome_stick_linewidth = 2
    max_cover_depth = 3
    """

    row_plot_dict, chr_site_dict, gap_length, max_row_length = divide_into_row(
        plot_chr_id_list, chr_length_dict, base_per_row)

    # make main axes x,y limit
    row_num = len(row_plot_dict)
    plot_size_range = [10, 110]
    x_y_lim = (
        (plot_size_range[0]-10, plot_size_range[1]+10), (1, (row_num+1)*row_spacing))

    # make fig & ax
    fig = plt.figure(figsize=fig_size)
    ax = fig.add_subplot(1, 1, 1)
    ax.set_xlim(x_y_lim[0])
    ax.set_ylim(x_y_lim[1])
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)

    chr_plot_dict = plot_genome_by_row(ax, row_plot_dict, chr_site_dict, max_row_length, chr_length_dict, plot_size_range,
                                       x_y_lim[1][1]-row_spacing, row_spacing, x_y_lim=x_y_lim, thickness=genome_stick_thickness, linewidth=genome_stick_linewidth, color=genome_stick_color)

    for chr_id in chr_plot_dict:
        (chr_genome_start,
         chr_genome_end), chr_genome_high, thickness = chr_plot_dict[chr_id]
        chr_ax = add_small_axes_on_other(
            ax, fig, (chr_genome_start, chr_genome_high + thickness), (chr_genome_end, chr_genome_high + thickness + 1.5))
        chr_ax.patch.set_facecolor('none')

        add_cover_plot(
            chr_ax, base_cover_chr_dict[chr_id], chr_length_dict[chr_id])

        chr_ax.set_xlim((1, chr_length_dict[chr_id]))
        chr_ax.set_ylim((0, max_cover_depth+0.5))
        chr_ax.yaxis.set_major_locator(ticker.MultipleLocator(base=1))
        chr_ax.get_xaxis().set_visible(False)
        chr_ax.spines['top'].set_visible(False)
        chr_ax.spines['right'].set_visible(False)
        chr_ax.spines['bottom'].set_visible(False)

        if not chr_genome_start == plot_size_range[0]:
            chr_ax.spines['left'].set_visible(False)
            chr_ax.get_yaxis().set_visible(False)

    plt.show()

# dotplot + cover plot


def add_contig_grid(ax, query_contig_length_dict, subject_contig_length_dict, q_top=None, s_top=None, grid_colors='k', linewidths=0.5, q_contig_list=None, s_contig_list=None):
    query_contig_coord = {}
    subject_contig_coord = {}

    # for query aka y axis h
    q_hline_y_site = []
    q_tick_local = []
    q_tick_label = []
    q_num = 0
    q_top_num = 0

    if q_contig_list is None:
        q_contig_list = list(query_contig_length_dict.keys())

    for i in q_contig_list:
        q_top_num += 1
        if q_top and q_top_num > q_top:
            continue
        if q_top_num == 1:
            top_q_len = query_contig_length_dict[i]
        query_contig_coord[i] = q_num
        q_num += query_contig_length_dict[i]
        q_hline_y_site.append(q_num)
        q_tick_local.append(q_num - query_contig_length_dict[i]/2)
        q_tick_label.append(i)

    q_hline_y_site = q_hline_y_site[:-1]

    ax.hlines(q_hline_y_site, 0, 1, transform=ax.get_yaxis_transform(),
              colors=grid_colors, linewidths=linewidths)

    # for subject aka x axis v
    s_vline_y_site = []
    s_tick_local = []
    s_tick_label = []
    s_num = 0
    s_top_num = 0

    if s_contig_list is None:
        s_contig_list = list(subject_contig_length_dict.keys())

    for i in s_contig_list:
        s_top_num += 1
        if s_top and s_top_num > s_top:
            continue
        if s_top_num == 1:
            top_s_len = subject_contig_length_dict[i]
        subject_contig_coord[i] = s_num
        s_num += subject_contig_length_dict[i]
        s_vline_y_site.append(s_num)
        s_tick_local.append(s_num - subject_contig_length_dict[i]/2)
        s_tick_label.append(i)

    s_vline_y_site = s_vline_y_site[:-1]

    ax.vlines(s_vline_y_site, 0, 1, transform=ax.get_xaxis_transform(),
              colors=grid_colors, linewidths=linewidths)

    ax.invert_yaxis()

    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    ax.spines['left'].set_visible(False)

    ax.tick_params(axis='both', which='both', bottom=False, top=False,
                   left=False, right=False, labeltop=True, labelleft=True)

    ax.set_xticks(s_tick_local)
    ax.set_xticklabels(s_tick_label)
    ax.set_yticks(q_tick_local)
    ax.set_yticklabels(q_tick_label)

    ax.xaxis.set_tick_params(
        which='major', labelrotation=90, labeltop=True, labelbottom=False)

    x_y_lim = ((0, s_num), (q_num, 0))
    ax.set_xlim(x_y_lim[0])
    ax.set_ylim(x_y_lim[1])

    return query_contig_coord, subject_contig_coord


def add_synteny_dot(ax, query_contig_coord, subject_contig_coord, synteny_block_dict, reverse=False, color='#617a95', linewidth=3):
    for id_tmp in synteny_block_dict:
        sb_tmp = synteny_block_dict[id_tmp]
        strand = sb_tmp.strand
        if not reverse:
            q_id = sb_tmp.q_chr
            s_id = sb_tmp.s_chr
            q_f = sb_tmp.query_from
            q_t = sb_tmp.query_to
            s_f = sb_tmp.subject_from
            s_t = sb_tmp.subject_to
        else:
            q_id = sb_tmp.s_chr
            s_id = sb_tmp.q_chr
            q_f = sb_tmp.subject_from
            q_t = sb_tmp.subject_to
            s_f = sb_tmp.query_from
            s_t = sb_tmp.query_to

        if q_id not in query_contig_coord or s_id not in subject_contig_coord:
            continue

        q_f = query_contig_coord[q_id] + q_f
        q_t = query_contig_coord[q_id] + q_t

        s_f = subject_contig_coord[s_id] + s_f
        s_t = subject_contig_coord[s_id] + s_t

        if strand == '+':
            ax.plot((s_f, s_t), (q_f, q_t), color, linewidth=linewidth)
        else:
            ax.plot((s_t, s_f), (q_f, q_t), color, linewidth=linewidth)


def small_cover_plot(ax, orientation, base_cover_chr_dict, contig_coord, max_cover_depth=5, facecolor='r'):

    rectangle_list = []
    for chr_id in base_cover_chr_dict:
        for depth in base_cover_chr_dict[chr_id]:
            for s, e in base_cover_chr_dict[chr_id][depth]:
                if chr_id not in contig_coord:
                    continue
                contig_coord_base = contig_coord[chr_id]
                ps, pe = s+contig_coord_base, e+contig_coord_base
                if orientation == 'h':
                    rect = Patches.Rectangle((ps, 0), pe-ps, depth)
                elif orientation == 'v':
                    rect = Patches.Rectangle((0, ps), depth, pe-ps)
                rectangle_list.append(rect)

    pc = PatchCollection(rectangle_list, facecolor=facecolor, edgecolor='None')
    ax.add_collection(pc)

    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    if orientation == 'h':
        ax.set_ylim((0, max_cover_depth+0.5))
        ax.yaxis.set_major_locator(ticker.MultipleLocator(base=1))
        ax.get_xaxis().set_visible(False)

    elif orientation == 'v':
        ax.set_xlim((0, max_cover_depth+0.5))
        ax.xaxis.set_major_locator(ticker.MultipleLocator(base=1))
        ax.get_yaxis().set_visible(False)


def synteny_dotplot(query_id, subject_id, synteny_block_dict, q_range_base_cover_chr_dict, s_range_base_cover_chr_dict, query_genome_fasta, subject_genome_fasta, q_top=None, s_top=None, reverse=False, max_cover_depth=4, save_file=None, q_contig_list=None, s_contig_list=None):
    # parameter
    fig_size = (20, 20)

    # chr length
    query_fasta_dict = read_fasta_by_faidx(query_genome_fasta)
    query_contig_length_dict = OrderedDict()
    for i in sorted(list(query_fasta_dict.keys()), key=lambda x: query_fasta_dict[x].len(), reverse=True):
        query_contig_length_dict[i] = query_fasta_dict[i].len()

    subject_fasta_dict = read_fasta_by_faidx(subject_genome_fasta)
    subject_contig_length_dict = OrderedDict()
    for i in sorted(list(subject_fasta_dict.keys()), key=lambda x: subject_fasta_dict[x].len(), reverse=True):
        subject_contig_length_dict[i] = subject_fasta_dict[i].len()

    # make fig & ax
    left, width = 0.1, 0.65
    bottom, height = 0.1, 0.65
    spacing = 0.02
    cover_plot_height = 0.04

    rect_scatter = [left, bottom, width, height]
    rect_histx = [left, bottom - cover_plot_height -
                  spacing, width, cover_plot_height]
    rect_histy = [left + width + spacing*1.5,
                  bottom, cover_plot_height, height]

    fig = plt.figure(figsize=fig_size)

    ax = fig.add_axes(rect_scatter)
    ax_histx = fig.add_axes(rect_histx, sharex=ax)
    ax_histy = fig.add_axes(rect_histy, sharey=ax)

    query_contig_coord, subject_contig_coord = add_contig_grid(
        ax, query_contig_length_dict, subject_contig_length_dict, q_top, s_top, q_contig_list=q_contig_list, s_contig_list=s_contig_list)
    add_synteny_dot(ax, query_contig_coord, subject_contig_coord,
                    synteny_block_dict, reverse=reverse)

    small_cover_plot(ax_histx, 'h', s_range_base_cover_chr_dict,
                     subject_contig_coord, max_cover_depth=max_cover_depth)
    # ax_histx.set_title(query_id,fontsize=20)
    small_cover_plot(ax_histy, 'v', q_range_base_cover_chr_dict,
                     query_contig_coord, max_cover_depth=max_cover_depth)
    # ax_histy.set_ylabel(subject_id,fontsize=20)

    ax.set_xlabel(subject_id, fontsize=20)
    ax.xaxis.set_label_position('bottom')
    ax.xaxis.set_label_coords(0.5, -0.01)
    ax.set_ylabel(query_id, fontsize=20)
    ax.yaxis.set_label_position('right')
    ax.yaxis.set_label_coords(1.01, 0.5)

    plt.show()

    if save_file:
        fig.savefig(save_file, format='pdf', facecolor='none',
                    edgecolor='none', bbox_inches='tight')


# cover stat plot
def cover_stat_plot(speci_list, pair_dict):
    fig = plt.figure(figsize=(20, 20))

    num = 1
    for i in speci_list:
        for j in speci_list:

            ax = fig.add_subplot(len(speci_list), len(speci_list), num)
            ax.xaxis.set_major_locator(ticker.MultipleLocator(base=1))
            ax.set_xlim(0, 7)
            ax.set_ylim(0, 1)

            if not num % len(speci_list) == 1 and num != 2:
                labels = [item.get_text() for item in ax.get_yticklabels()]

                empty_string_labels = ['']*len(labels)
                ax.set_yticklabels(empty_string_labels)

            if num < len(speci_list)*(len(speci_list)-1):
                labels = [item.get_text() for item in ax.get_xticklabels()]

                empty_string_labels = ['']*len(labels)
                ax.set_xticklabels(empty_string_labels)

            if i == j:
                text = ax.text(3.5, 0.5, i, ha='center', va='center', size=20)
                text.set_path_effects([path_effects.Normal()])
                ax.spines['top'].set_visible(False)
                ax.spines['right'].set_visible(False)
                ax.spines['left'].set_visible(False)
                ax.spines['bottom'].set_visible(False)
                ax.get_xaxis().set_visible(False)
                ax.get_yaxis().set_visible(False)
                num += 1
                continue
            else:
                sp_pair = (i, j)
                sp_pair_r = (j, i)
                if sp_pair in pair_dict:
                    q_no_zero_sum_ratio, q_no_zero_ratio, s_no_zero_sum_ratio, s_no_zero_ratio = pair_dict[
                        sp_pair]['gene_cover_stat']
                    plot_no_zero_sum_ratio, plot_no_zero_ratio = s_no_zero_sum_ratio, s_no_zero_ratio
                else:
                    q_no_zero_sum_ratio, q_no_zero_ratio, s_no_zero_sum_ratio, s_no_zero_ratio = pair_dict[
                        sp_pair_r]['gene_cover_stat']
                    plot_no_zero_sum_ratio, plot_no_zero_ratio = q_no_zero_sum_ratio, q_no_zero_ratio

                text = ax.text(4.5, 0.85, "%.2f%%" %
                               (plot_no_zero_sum_ratio*100), size=12)
                ax.bar(list(range(1, len(plot_no_zero_ratio)+1)),
                       plot_no_zero_ratio)
                num += 1

    plt.show()


if __name__ == "__main__":

    # make a big synteny compare between two genome

    query_id = 'Gel'
    query_contig_list_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Gel.chr.id'
    query_gff_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Gel.gff3'
    query_fasta_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.genome.v2.0.fasta'

    subject_id = 'Aof'
    subject_contig_list_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Aof.chr.id'
    subject_gff_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Aof.gff3'
    subject_fasta_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Asparagus_officinalis/T4686N0.genome.fasta'

    query_vs_subject_collinearity_csv_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Aof_vs_Gel_h/mcscanx.collinearity.csv'

    query_plot_chr_id_list = read_list_file(query_contig_list_file)
    subject_plot_chr_id_list = read_list_file(subject_contig_list_file)

    plot_size_range = [10, 110]
    x_y_lim = ((plot_size_range[0]-10, plot_size_range[1]+10), (1, 10))

    fig = plt.figure(figsize=(20, 10))
    ax = fig.add_subplot(1, 1, 1)

    two_genome_synteny_block_plot(ax, plot_size_range, x_y_lim, query_id, query_plot_chr_id_list, query_gff_file, query_fasta_file,
                                  subject_id, subject_plot_chr_id_list, subject_gff_file, subject_fasta_file, query_vs_subject_collinearity_csv_file)

    ax.set_xlim(x_y_lim[0])
    ax.set_ylim(x_y_lim[1])

    plt.show()

    # plot WGD synteny genome cover
    from api.xuyuxing.comp_genome.mcscan import WGD_check_pipeline

    query_species = 'Ash'
    query_gff3 = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Ash.gff3'
    query_genome_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Apostasia_shenzhenica/T1088818N0.genome.fasta'
    subject_species = 'Gel'
    subject_gff3 = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Gel.gff3'
    subject_genome_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.genome.v2.0.fasta'
    mcscanx_collinearity_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Ash_vs_Gel_h/mcscanx.collinearity'

    q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict, s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict = WGD_check_pipeline(
        mcscanx_collinearity_file, query_species, query_gff3, query_genome_fasta, subject_species, subject_gff3, subject_genome_fasta)

    sp_id = 'Gel'
    contig_list_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Gel.chr.id'
    genome_fasta_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.genome.v2.0.fasta'

    plot_chr_id_list = read_list_file(contig_list_file)
    fasta_dict = read_fasta_by_faidx(genome_fasta_file)
    chr_length_dict = {i: fasta_dict[i].len() for i in fasta_dict}

    genome_cover_plot(plot_chr_id_list, chr_length_dict,
                      s_range_base_cover_chr_dict)

    # plot dotplot
    from api.xuyuxing.comp_genome.mcscan import WGD_check_pipeline

    query_species = 'Ash'
    query_gff3 = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Ash.gff3'
    query_genome_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Apostasia_shenzhenica/T1088818N0.genome.fasta'
    subject_species = 'Gel'
    subject_gff3 = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Gel.gff3'
    subject_genome_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.genome.v2.0.fasta'
    mcscanx_collinearity_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Ash_vs_Gel_h/mcscanx.collinearity'

    q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict, s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict = WGD_check_pipeline(
        mcscanx_collinearity_file, query_species, query_gff3, query_genome_fasta, subject_species, subject_gff3, subject_genome_fasta)

    synteny_dotplot('Ash', 'Gel', synteny_block_dict, q_range_base_cover_chr_dict, s_range_base_cover_chr_dict,
                    query_genome_fasta, subject_genome_fasta, q_top=200, s_top=18, reverse=False, max_cover_depth=4)



import pysam
import os
from lib.base.base_function import cmd_run
import time 
from lib.seq.seq_base import read_fastq_big
import sys
import lib.file_parser.sqlite_command as sc
from lib.file_parser.fileIO import read_list_file

__author__ = 'Yuxing Xu'


def read_fasta(fasta_file):
    fasta_dict = {}
    with open(fasta_file, 'r') as f:
        all_str = f.read()
        all_str_splited_list = all_str.split('>')
        all_str_splited_list = all_str_splited_list[1:]

        for i in all_str_splited_list:
            contig_info_list = i.split('\n')
            contig_id = contig_info_list[0].split()[0]
            seq_list = contig_info_list[1:]
            seq = ''
            for j in seq_list:
                seq = seq + j
            fasta_dict[contig_id] = seq

    return fasta_dict


# def read_fasta2(fasta_file):
#     fasta_dict = {}
#
#     for record in SeqIO.parse(fasta_file, "fasta"):
#         contig_id = record.id
#         contig_seq = str(record.seq)
#
#         fasta_dict[contig_id] = contig_seq
#
#     return fasta_dict


def read_bam(bam_file):
    bf = pysam.AlignmentFile(bam_file, 'r')
    for r in bf.fetch(until_eof=True):
        pass

        if len(read_tmp_record) == 0 or r.query_name == read_tmp_record[-1].query_name:
            read_tmp_record.append(r)
            continue
        else:
            yield read_tmp_record
            read_tmp_record = [r]
    if len(read_tmp_record) != 0:
        yield read_tmp_record
    bf.close()

def check_md5(dir_path):
    dir_path = os.path.abspath(dir_path)

    file_dir_list = os.listdir(dir_path)
    for tmp_name in file_dir_list:
        tmp_name_full_path = dir_path + "/" + tmp_name
        if os.path.isdir(tmp_name_full_path):
            check_md5(tmp_name_full_path)
        else:
            tmp_list = tmp_name.split("_")
            if len(tmp_list) > 1:
                if tmp_list[0] == 'MD5':
                    cmd_string = "md5sum -c %s" % tmp_name
                    flag, output, error = cmd_run(cmd_string, cwd=dir_path, retry_max=5, silence=True)
                    print (output)

def decompressed_gz(dir_path):
    dir_path = os.path.abspath(dir_path)

    file_dir_list = os.listdir(dir_path)
    for tmp_name in file_dir_list:
        tmp_name_full_path = dir_path + "/" + tmp_name
        if os.path.isdir(tmp_name_full_path):
            decompressed_gz(tmp_name_full_path)
        else:
            tmp_list = tmp_name.split(".")
            if len(tmp_list) > 1:
                if tmp_list[-1] == 'gz':
                    cmd_string = "gunzip %s" % tmp_name
                    flag, output, error = cmd_run(cmd_string, cwd=dir_path, retry_max=5, silence=True)

def copy_gz(dir_path,target_dir):
    dir_path = os.path.abspath(dir_path)

    file_dir_list = os.listdir(dir_path)
    for tmp_name in file_dir_list:
        tmp_name_full_path = dir_path + "/" + tmp_name
        if os.path.isdir(tmp_name_full_path):
            copy_gz(tmp_name_full_path,target_dir)
        else:
            tmp_list = tmp_name.split(".")
            if len(tmp_list) > 1:
                if tmp_list[-1] == 'gz':
                    cmd_string = "cp %s %s" % (tmp_name,target_dir)
                    flag, output, error = cmd_run(cmd_string, cwd=dir_path, retry_max=5, silence=True)

def readsID_calling(dup_id,all_id,not_dup_id):
    dup_id_dict = {}
    with open(dup_id, 'r') as f:
        for each_line in f:
            each_line = each_line.strip()
            dup_id_dict[each_line] = ""
    
    # start_time = time.time()
    # num=0
    with open(not_dup_id, 'w') as o:
        with open(all_id, 'r') as f:
            for each_line in f:
                # num = num +1

                # round_time = timr.time()
                # if round_time-start_time >2
                #    start_time= round_time 
                    #  print(num)
                each_line = each_line.strip()
                if not each_line in dup_id_dict:
                    o.write(each_line + "\n")

# valid_aln_list = [alignedsegment for alignedsegment in read_aln_list if not (alignedsegment.is_secondary or alignedsegment.is_supplementary)]
# upper code = following code 
# valid_aln_list = []
# for alignedsegment in read_aln_list:
#     if not (alignedsegment.is_secondary or alignedsegment.is_supplementary):
#          valid_aln_list.append(alignedsegment)

# get all of the reads ID from clean fastaq files


def get_read_name_list_from_fastq(fastq_file_name, output_file_name):
   
    with open(output_file_name, 'w') as f:
        for record in read_fastq_big(fastq_file_name,gzip_flag=True):
            f.write(record[0].seqname_short() + "\n")

    return output_file_name


def index_bam_file(bam_file):
    bamfile = pysam.AlignmentFile(bam_file, 'rb')
    name_indexed = pysam.IndexedReads(bamfile)
    name_indexed.build()
    header = bamfile.header.copy()
    return name_indexed, header


def extract_aln_by_reads(reads_name, name_indexed):
    output_list = []
    try:
        name_indexed.find(reads_name)
    except KeyError:
        pass
    else:
        iterator = name_indexed.find(reads_name)
        output_list = []
        for x in iterator:
            output_list.append(x)
    
    return output_list

def get_top_M_score(read_aln_list):
    top_M = 0
    
    for aln in read_aln_list:
        cigar_list = aln.cigar
        TM = sum([i[1] for i in cigar_list if i[0] == 0])
        DM = sum([i[1] for i in cigar_list if not (i[0] == 0 or i[0] == 3)])

        M = TM - DM

        if M > top_M:
            top_M = M
    
    return top_M


def get_best_alignment_match_number(read_aln_list):
    valid_aln_list = [alignedsegment for alignedsegment in read_aln_list if not (alignedsegment.is_secondary or alignedsegment.is_supplementary)]

    # get 1-end
    read1_aln_list = [alignedsegment for alignedsegment in valid_aln_list if alignedsegment.is_read1]

    # get 2-end
    read2_aln_list = [alignedsegment for alignedsegment in valid_aln_list if alignedsegment.is_read2]

    read1_top_M = get_top_M_score(read1_aln_list)
    read2_top_M = get_top_M_score(read2_aln_list)

    M_score = read1_top_M + read2_top_M

    return M_score


def get_tag(read_aln_list1, read_aln_list2, tag1, tag2):
    M1 = get_best_alignment_match_number(read_aln_list1)
    M2 = get_best_alignment_match_number(read_aln_list2)

    if M1 > M2:
        return tag1, M1, M2
    elif M2 > M1:
        return tag2, M1, M2
    else:
        return "X", M1, M2

def write_aln(read_name, name_indexed, header, sam_output_file):
    aln_list = extract_aln_by_reads(read_name, name_indexed)

    out = pysam.Samfile(sam_output_file, 'wb', header=header)
    for aln in aln_list:
        out.write(aln)

def get_tag_new(reads_id, db_file):
    table_id = sum([int(i) for i in reads_id.split(":")[-3:]]) % 100
    table_id = "table" + str(table_id)

    reads_id_get, tag_get = sc.sqlite_select(db_file, table_id, ['reads_id', 'tag'], 'reads_id', (reads_id,))[0]
    
    return tag_get

def get_tag_many(reads_id_list, db_file):

    table_reads_dict = {}
    for reads_id in reads_id_list:
        table_id = sum([int(i) for i in reads_id.split(":")[-3:]]) % 100
        table_id = "table" + str(table_id)
        if table_id not in table_reads_dict:
            table_reads_dict[table_id] = []
        table_reads_dict[table_id].append(reads_id)

    tag_dict = {}
    for table_id in table_reads_dict:
        info = sc.sqlite_select(db_file, table_id, ['reads_id', 'tag'], 'reads_id', table_reads_dict[table_id])
        for reads_id_get, tag_get in info:
            tag_dict[reads_id_get] = tag_get
    
    return tag_dict


if __name__ == '__main__':
    import argparse

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='TestTools',
    )

    subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")

    # argparse for SubGenomeExtracter
    parser_a = subparsers.add_parser('SubGenomeExtracter',
                                     help='extract a genome fragment\n',
                                     description='')

    parser_a.add_argument('genome_fasta_file', type=str, help='raw genome fasta file')
    parser_a.add_argument('output_fasta_file', type=str, help='output fasta file')
    parser_a.add_argument('table_file', type=str, help='a table which include info of locals, (Chr1  1   100)')
    parser_a.add_argument("-e", "--expand_range", help="length of expand range", type=int, default=0)

    # argparse for ReadsCleanTools
    parser_a = subparsers.add_parser('ReadsCleanTools',
                                     help='Clean bad mapped reads\n',
                                     description='')

    parser_a.add_argument('reads_bam_file', type=str, help='mapping reads bam file')
    parser_a.add_argument('output_fasta_file', type=str,
                          help='a clean fasta file which exclude bad mapping qualities reads info of flower-related')
    parser_a.add_argument('threshold', type=float,
                          help='threshold')

    # argparse for CheckMD5
    parser_a = subparsers.add_parser('CheckMD5',
                                     help='check MD5 value rightness \n',
                                     description='')

    parser_a.add_argument('dir_path', type=str, help='the absolute path of dir')

    # argparse for decompressed_gz
    parser_a = subparsers.add_parser('decompressed_gz',
                                     help='decompressed all gz file \n',
                                     description='')

    parser_a.add_argument('dir_path', type=str, help='the absolute path of dir')

    # argparse for copy_gz
    parser_a = subparsers.add_parser('copy_gz',
                                     help='copy all gz file \n',
                                     description='')

    parser_a.add_argument('dir_path', type=str, help='the absolute path of dir')
    parser_a.add_argument('target_dir', type=str, help='the target stored dir')

    # argparse for rm_cleandata_file
    parser_a = subparsers.add_parser('rm_cleandata_file',
                                     help='rm all cleandata file \n',
                                     description='')

    parser_a.add_argument('dir_path', type=str, help='the absolute path of dir')

    # argparse for readsID_calling
    parser_a = subparsers.add_parser('readsID_calling',
                                     help='calling out the no duplicated reads \n',
                                     description='')

    parser_a.add_argument('dup_id', type=str, help='the duplicated reads id file')
    parser_a.add_argument('all_id', type=str, help='the all reads id file')
    parser_a.add_argument('not_dup_id', type=str, help='the no duplicated reads id file')

    # argparse for MappedReadsComparing
    parser_a = subparsers.add_parser('MappedReadsComparing',
                                     help='comparing the mapped reads CIgar from two reference genome \n',
                                     description='')

    parser_a.add_argument('fastq_file', type=str, help='the clean reads file')
    parser_a.add_argument('mapped_bam_file_1', type=str, help='the bam file 1 from reference genome A188')
    parser_a.add_argument('mapped_bam_file_2', type=str, help='the bam file 2 from reference genome B73')
    parser_a.add_argument('reads_id_file', type=str, help='the reads id list from fastq file')
    parser_a.add_argument('tag1', type=str, help='A188 flag')
    parser_a.add_argument('tag2', type=str, help='B73 flag')
    parser_a.add_argument('tag_file', type=str, help='the output tag file')
    
    # ConvertTagfileToDBfile
    parser_a = subparsers.add_parser('ConvertTagfileToDBfile',
                                     help='converting the tag files to databasefiles \n',
                                     description='')
    parser_a.add_argument('tag_file', type=str, help='the tag file')
    parser_a.add_argument('db_file', type=str, help='the database file')
    
    # ConvertReadsidfileToDBfile
    parser_a = subparsers.add_parser('ConvertReadsidfileToDBfile',
                                     help='converting the reads id files to databasefiles \n',
                                     description='')
    parser_a.add_argument('readsid_file', type=str, help='the reads id file')
    parser_a.add_argument('db_file', type=str, help='the database file')


    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # ccommand detail

    # SubGenomeExtracter
    if args_dict["subcommand_name"] == "SubGenomeExtracter":
        """
        class abc():
            pass

        args = abc()

        args.genome_fasta_file = '/lustre/home/macanrong/Work/Nicotiana_sylvestris/GCF_000393655.1_Nsyl_genomic.fna.fasta'
        args.output_fasta_file = '/lustre/home/xuyuxing/tmp.file'
        args.table_file = '/lustre/home/macanrong/Work/Nicotiana_sylvestris/Flow.gene.loci.clean'
        args.expand_range = 3000
        """

        # read table file
        table_info_dict = {}
        num = 0
        with open(args.table_file, 'r') as f:
            for each_line in f:
                num = num + 1
                info_list = each_line.split()
                table_info_dict[num] = info_list

        # read genome file
        fasta_dict = read_fasta(args.genome_fasta_file)

        # extract
        output_dict = {}
        for id in table_info_dict:
            contig_id, start_point, end_point = table_info_dict[id]
            start_point = int(start_point)
            end_point = int(end_point)

            contig_length = len(fasta_dict[contig_id])

            expand_start_point = max(1, start_point - args.expand_range)
            expand_end_point = min(contig_length, end_point + args.expand_range)

            get_sub_genome_seq = fasta_dict[contig_id][expand_start_point - 1:expand_end_point]
            get_sub_genome_seq = get_sub_genome_seq.upper()

            # output_id = contig_id + ":" + str(expand_start_point) + "-" + str(expand_end_point)
            output_id = "%s:%d-%d" % (contig_id, expand_start_point, expand_end_point)

            output_dict[output_id] = get_sub_genome_seq

        with open(args.output_fasta_file, 'w') as f:
            for output_id in output_dict:
                output_seq = output_dict[output_id]
                f.write(">%s\n%s\n" % (output_id, output_seq))

    elif args_dict["subcommand_name"] == "ReadsCleanTools":
        """
        class abc():
            pass

        args = abc()

        args.reads_bam_file = '/lustre/home/macanrong/Work/Nicotiana_sylvestris/Nsyl.m64033_200313_001421.bam'
        args.output_fasta_file = '/lustre/home/macanrong/Work/Nicotiana_sylvestris/Nsyl.m64033_200313_001421.fasta'
        args.threshold = 0.2
        """

        # read bam file

        read_appear_counter = {}

        bf = pysam.AlignmentFile(args.reads_bam_file, 'r')

        with open(args.output_fasta_file, 'w') as f:
            for r in bf.fetch(until_eof=True):
                read_name = str(r.query_name)
                aln_length = int(r.query_alignment_length)
                aln_seq = str(r.query_alignment_sequence)

                ref_name = str(r.reference_name)
                start, end = ref_name.split(":")[1].split("-")
                ref_length = int(end) - int(start) + 1

                if aln_length / ref_length > args.threshold:

                    # reads name repeat check
                    if read_name in read_appear_counter:
                        read_appear_counter[read_name] = read_appear_counter[read_name] + 1
                    else:
                        read_appear_counter[read_name] = 1

                    read_name_mod = read_name + "_" + str(read_appear_counter[read_name])

                    f.write(">%s\n%s\n" % (read_name_mod, aln_seq))

    # check_md5
    elif args_dict["subcommand_name"] == "CheckMD5":
        """
        class abc():
            pass

        args = abc()

        args.dir_path = '/lustre/home/macanrong/Database/maize_F1/1.rawdata'
        """

        # print(args.dir_path)
        # print(args.output_MD5_file)

        check_md5_list = check_md5(args.dir_path)
        print(check_md5_list)


    #decompressed_gz
    elif args_dict["subcommand_name"] == "decompressed_gz":
        """
        class abc():
            pass

        args = abc()

        args.dir_path = '/lustre/home/macanrong/Database/maize_F1/1.rawdata'
        """
        decompressed_gz(args.dir_path)

    # copy_gz
    elif args_dict["subcommand_name"] == "copy_gz":
        """
        class abc():
            pass

        args = abc()

        args.dir_path = '/lustre/home/macanrong/Database/maize_F1/1.rawdata'
        args.target_dir = '/lustre/home/macanrong/Database/maize_F1/cleandata'
        """
        copy_gz(args.dir_path,args.target_dir)

    # rm_cleandata_file
    elif args_dict["subcommand_name"] == "rm_cleandata_file":
        """
        class abc():
            pass

        args = abc()

        args.dir_path = '/lustre/home/macanrong/Database/maize_F1/1.rawdata'
        args.target_dir = '/lustre/home/macanrong/Database/maize_F1/cleandata'
        """
        rm_cleandata_file(args.dir_path)

    # readsID_calling
    elif args_dict["subcommand_name"] == "readsID_calling":
        """
        class abc():
        pass

        args = abc()

        args.dup_id = '/lustre/home/macanrong/Database/maize_F101/cleandata/deduplic_dir/M21-1.dup.id'
        args.all_id = '/lustre/home/macanrong/Database/maize_F101/cleandata/deduplic_dir/M21-1.all.id'
        args.not_dup_id = '/lustre/home/macanrong/Database/maize_F101/cleandata/deduplic_dir/M21-1.not_dup.id'
        """

        readsID_calling(args.dup_id, args.all_id, args.not_dup_id)

    # MappedReadsComparing
    elif args_dict["subcommand_name"] == "MappedReadsComparing":
        # get all of the reads ID from clean fastaq files
        """
        class abc():
        pass

        args = abc()
        args.fastq_file = '/lustre/home/macanrong/Work/maize_F1/tophat2/test/20000.fq.gz'
        args.reads_id_file = '/lustre/home/macanrong/Work/maize_F1/tophat2/test/20000.id'
        args.mapped_bam_file_1 ='/lustre/home/macanrong/Work/maize_F1/tophat2/test/accepted_hits.A188.bam'
        args.mapped_bam_file_2 ='/lustre/home/macanrong/Work/maize_F1/tophat2/test/accepted_hits.B73.bam'
        args.tag1 = "A188"
        args.tag2 = "B73"
        args.tag_file = "/lustre/home/macanrong/Work/maize_F1/tophat2/test/20000.id.tag"
        """


        def MappedReadsComparing_main(args):
            reads_id_file= get_read_name_list_from_fastq(args.fastq_file, args.reads_id_file)


            # indexed the bam file
            name_indexed_1, header1 = index_bam_file(args.mapped_bam_file_1)
            name_indexed_2, header2 = index_bam_file(args.mapped_bam_file_2)

            # remove the None element
            with open(args.tag_file, 'w') as w:
                with open(args.reads_id_file, 'r') as f:
                    for each_line in f:
                        read_name = each_line.strip()
                
                        # extracting the alignmentseqments
                        read_aln_list_1= extract_aln_by_reads(read_name, name_indexed_1)
                        read_aln_list_2= extract_aln_by_reads(read_name, name_indexed_2)
                
                        tag, M1 , M2  = get_tag(read_aln_list_1, read_aln_list_2, args.tag1, args.tag2)

                        w.write("%s\t%s\t%d\t%d\n" % (read_name, tag, M1, M2))

    # ConvertTagfileToDBfile
    elif args_dict["subcommand_name"] == "ConvertTagfileToDBfile":
        # converting the tag files into the database files
        """
        class abc():
        pass

        args = abc()
        args.tag_file = "/lustre/home/macanrong/Work/maize_F1/tophat2/M46.tag"
        args.db_file = "/lustre/home/xuyuxing/M46.db"
        """

        sys.path.append("/lustre/home/xuyuxing/python_project/Genome_work_tools")

        table_columns_dict = { "table" + str(i) : ["reads_id", "tag", "A_score", "B_score"] for i in range(0,100) }


        sc.init_sql_db_many_table(db_file, table_columns_dict, remove_old_db=True)

        # num = 0

        waitting_dict = {}

        with open(args.tag_file, "r") as f:
            for each_line in f:
                # num += 1
                # if num > 50000:
                #     break
                each_line = each_line.strip()
                info = each_line.split()
                reads_id = info[0]
                reads_tag = info[1]
                reads_sample1 = int(info[2])
                reads_sample2 = int(info[3])

                table_id = sum([int(i) for i in reads_id.split(":")[-3:]]) % 100
                table_id = "table" + str(table_id)

            if table_id not in waitting_dict:
                waitting_dict[table_id] = []
                waitting_dict[table_id].append((reads_id, reads_tag, reads_sample1, reads_sample2))

            if len(waitting_dict[table_id]) > 10000:
                sc.sqlite_write(waitting_dict[table_id], args.db_file, table_id, ["reads_id", "tag", "A_score", "B_score"])
                waitting_dict[table_id] = []

        for table_id in waitting_dict:
            if len(waitting_dict[table_id]) > 0:
                sc.sqlite_write(waitting_dict[table_id], args.db_file, table_id, ["reads_id", "tag", "A_score", "B_score"])
                waitting_dict[table_id] = []

        for table_id in table_columns_dict:
            sc.build_index(args.db_file, table_id, 'reads_id')


    # ConvertReadsidfileToDBfile
    elif args_dict["subcommand_name"] == "ConvertReadsidfileToDBfile":
        # converting the reads ID files into the database files
        """
        class abc():
        pass

        args = abc()
        args.reads_id = "/lustre/home/macanrong/Work/maize_F1/tophat2/test/M9-1.readsid.file"
        args.db_file = "/lustre/home/macanrong/Work/maize_F1/tophat2/test/M9-1.db"
        """

        reads_id_list = read_list_file(args.reads_id)

        tag_list = [get_tag_new(i, args.db_file) for i in reads_id_list]
        tag_dict = get_tag_many(reads_id_list, args.db_file)

        















"""
This file work for Mercator output

Mercator4 v2.0 x (July 2019)

https://www.plabipd.de/portal/web/guest/mercator4
"""
from lib.common.fileIO import tsv_file_dict_parse
import re
from collections import OrderedDict
import copy


class MapmanFunction(object):
    def __init__(self, bin_code, name=None, terminal=None, sons=None):
        self.bin_code = bin_code
        self.name = name
        # self.description = description
        self.terminal = terminal
        self.sons = sons

        self.depth = len(get_ancestors_id_list(bin_code)) + 1
        self.ancestors = get_ancestors_id_list(bin_code)
        self.parent = get_parent_id(bin_code)

    def __str__(self):
        return '%s: %s (%d)' % (self.bin_code, self.name, self.depth)

    __repr__ = __str__


class MapmanTree(object):
    def __init__(self, mercator_results_file):
        mercator_info_dict = tsv_file_dict_parse(mercator_results_file)

        mercator_dict = {}
        parent_dict = {}
        for i in mercator_info_dict:
            bin_code = re.sub("'", "", mercator_info_dict[i]['BINCODE'])
            name = re.sub(
                "'", "", mercator_info_dict[i]['NAME'].split(".")[-1])
            # description = re.sub("'","",mercator_info_dict[i]['DESCRIPTION'])
            mercator_dict[bin_code] = MapmanFunction(bin_code, name=name)
            parent_dict[bin_code] = mercator_dict[bin_code].parent

        parent_hash_dict = {}
        for i in parent_dict:
            j = parent_dict[i]
            if j:
                if j not in parent_hash_dict:
                    parent_hash_dict[j] = []
                parent_hash_dict[j].append(i)

        for bin_code in mercator_dict:
            if bin_code not in parent_hash_dict:
                mercator_dict[bin_code].terminal = True
                mercator_dict[bin_code].sons = []
            else:
                mercator_dict[bin_code].terminal = False
                mercator_dict[bin_code].sons = sorted(
                    parent_hash_dict[bin_code])

        self.mercator_results_file = mercator_results_file

        for bin_code in mercator_dict:
            mercator_dict[bin_code].sons = [mercator_dict[i]
                                            for i in mercator_dict[bin_code].sons]
            mercator_dict[bin_code].ancestors = [mercator_dict[i]
                                                 for i in mercator_dict[bin_code].ancestors]
            if mercator_dict[bin_code].parent:
                mercator_dict[bin_code].parent = mercator_dict[mercator_dict[bin_code].parent]

        self.mercator_dict = OrderedDict()
        for i in sorted(list(mercator_dict.keys())):
            self.mercator_dict[i] = mercator_dict[i]

    def get(self, bin_code):
        return self.mercator_dict[bin_code]

    def browse(self, depth=None):
        for i in self.mercator_dict:
            if depth:
                if self.get(i).depth == depth:
                    yield self.mercator_dict[i]
            else:
                yield self.mercator_dict[i]

    def load_annotation(self, gene_bincode_map):
        for i in self.mercator_dict:
            self.mercator_dict[i].genes = []

        for g_id, bin_code in gene_bincode_map:
            self.mercator_dict[bin_code].genes.append(g_id)

    def get_genes(self, bin_code):
        gene_list = self.mercator_dict[bin_code].genes
        for son in self.mercator_dict[bin_code].sons:
            son_bin_code = son.bin_code
            gene_list.extend(self.get_genes(son_bin_code))

        gene_list = list(set(gene_list))

        if '' in gene_list:
            gene_list.remove('')

        return gene_list


def get_parent_id(bincode):
    parent_id = ".".join(bincode.split(".")[:-1])
    if parent_id == '':
        return None
    else:
        return parent_id


def get_ancestors_id_list(bincode):
    ancestors_id_list = []
    lastcode = bincode
    flag = True
    while flag:
        parent_id = get_parent_id(lastcode)
        if parent_id:
            ancestors_id_list.append(parent_id)
            lastcode = parent_id
        else:
            flag = False
    return sorted(ancestors_id_list)


def get_gene_bincode_map(mercator_results_file):
    mercator_info_dict = tsv_file_dict_parse(mercator_results_file)
    gene_bincode_map = []
    for i in mercator_info_dict:
        bin_code = re.sub("'", "", mercator_info_dict[i]['BINCODE'])
        gene_id = re.sub(
            "'", "", mercator_info_dict[i]['IDENTIFIER'])
        gene_bincode_map.append((gene_id, bin_code))
    return gene_bincode_map


def load_annotation(mapman, gene_bincode_map):
    output_mapman = copy.deepcopy(mapman)
    output_mapman.load_annotation(gene_bincode_map)
    return output_mapman


if __name__ == "__main__":
    mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Dendrobium_catenatum/T906689N0.gene_model.protein.Mercator.results.txt'

    mapman = MapmanTree(mercator_results_file)

    Dca_mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Dendrobium_catenatum/T906689N0.gene_model.protein.Mercator.results.txt'
    Gel_mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/T91201N0.gene_model.protein.Mercator.results.txt'
    Cau_mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Cuscuta_australis/T267555N0.gene_model.protein.Mercator.results.txt'

    Dca_mapman = load_annotation(
        mapman, get_gene_bincode_map(Dca_mercator_results_file))
    Gel_mapman = load_annotation(
        mapman, get_gene_bincode_map(Gel_mercator_results_file))
    Cau_mapman = load_annotation(
        mapman, get_gene_bincode_map(Cau_mercator_results_file))



from Bio import Phylo
from lib.common.evolution.tree_operate import lookup_by_names, add_clade_name, draw_ascii, get_MRCA_from_list, is_ancestors_of
from lib.common.os import multiprocess_running
from api.xuyuxing.evolution.PAML_tools import quick_get_dNdS
from itertools import combinations


gene_tree_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/OroWGD/time/tree/treebest/OG0005933/tree.nhx'
species_tree_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/OroWGD/time/species.tree.txt'
target_taxon_node = 'Lamiales'

gene_tree = Phylo.read(gene_tree_file, 'newick')
gene_tree = add_clade_name(gene_tree)
gene_tree_node_dict = lookup_by_names(gene_tree)
gene_list = [i.name for i in gene_tree.get_terminals()]

species_tree = Phylo.read(species_tree_file, 'newick')
species_tree = add_clade_name(species_tree)
species_tree_node_dict = lookup_by_names(species_tree)

map_dict = {}
for i in gene_list:
    j, species = i.split("_")
    map_dict[i] = species



def get_duplication_and_speciation_node(gene_tree, species_tree, map_dict, target_taxon_node):
    gene_tree_node_dict = lookup_by_names(gene_tree)
    species_tree_node_dict = lookup_by_names(species_tree)

    leaf_test_flag = species_tree_node_dict[target_taxon_node].is_terminal()

    DS_dict = {'D': [], 'S': []}

    for c_id in gene_tree_node_dict:
        clade = gene_tree_node_dict[c_id]

        if not clade.is_terminal():
            # get species_node
            sp_list_in_clade = list(
                set([map_dict[i.name] for i in clade.get_terminals()]))
            species_node_now = get_MRCA_from_list(
                [species_tree_node_dict[i] for i in sp_list_in_clade], species_tree).name

            if species_node_now != target_taxon_node:
                continue

            # get duplication or speciation
            D_flag = False

            for s1, s2 in combinations(clade.clades, 2):
                s1_leaf_list = [i.name for i in s1.get_terminals()]
                s2_leaf_list = [i.name for i in s2.get_terminals()]
                s1_species_list = list(
                    set([map_dict[i] for i in s1_leaf_list]))
                s2_species_list = list(
                    set([map_dict[i] for i in s2_leaf_list]))

                if len(set(s1_species_list) & set(s2_species_list)) > 0:
                    D_flag = True
            
            if D_flag:
                DS_dict['D'].append(c_id)
            else:
                DS_dict['S'].append(c_id)

        else:
            if leaf_test_flag:
                if map_dict[c_id] == target_taxon_node:
                    DS_dict['S'].append(c_id)

    return DS_dict


def check_duplication_for_one_tree_one_node(gene_tree, species_tree, map_dict, test_node, if_tandom_function):
    gene_tree_node_dict = lookup_by_names(gene_tree)
    species_tree_node_dict = lookup_by_names(species_tree)

    leaf_test_flag = species_tree_node_dict[test_node].is_terminal()

    DS_dict = {'D': [], 'S': []}

    for c_id in gene_tree_node_dict:
        clade = gene_tree_node_dict[c_id]

        if not clade.is_terminal():
            if not len(clade.clades) > 2:
                # get species_node
                sp_list_in_clade = list(
                    set([map_dict[i.name] for i in clade.get_terminals()]))
                species_node_now = get_MRCA_from_list(
                    [species_tree_node_dict[i] for i in sp_list_in_clade], species_tree).name

                if species_node_now != test_node:
                    continue

                # get duplication or speciation
                D_flag = False
                S_flag = False

                s1 = clade.clades[0]
                s2 = clade.clades[1]
                s1_leaf_list = [i.name for i in s1.get_terminals()]
                s2_leaf_list = [i.name for i in s2.get_terminals()]
                s1_species_list = list(
                    set([map_dict[i] for i in s1_leaf_list]))
                s2_species_list = list(
                    set([map_dict[i] for i in s2_leaf_list]))

                if len(set(s1_species_list) & set(s2_species_list)) > 0:
                    DS_dict['D'].append(c_id)
                    DS_dict['S'].append(c_id)
                else:
                    DS_dict['S'].append(c_id)

        else:
            if leaf_test_flag:
                if map_dict[c_id] == test_node:
                    DS_dict['S'].append(c_id)

    # rm overlap
    overlaped_s_c_id = []
    for s_c_id in DS_dict['S']:
        for d_c_id in DS_dict['D']:
            if s_c_id == d_c_id:
                overlaped_s_c_id.append(s_c_id)
            if is_ancestors_of(gene_tree_node_dict[d_c_id], gene_tree_node_dict[s_c_id], gene_tree):
                overlaped_s_c_id.append(s_c_id)

    DS_dict['S'] = list(set(DS_dict['S']) - set(overlaped_s_c_id))

    # filter tandom repeat
    tandom_c_id = []
    for d_c_id in DS_dict['D']:
        clade = gene_tree_node_dict[d_c_id]
        s1 = clade.clades[0]
        s2 = clade.clades[1]
        s1_leaf_list = [i.name for i in s1.get_terminals()]
        s2_leaf_list = [i.name for i in s2.get_terminals()]

        # make leaf_name_dict
        leaf_name_dict = {}
        for i in s1_leaf_list:
            sp = map_dict[i]
            if sp not in leaf_name_dict:
                leaf_name_dict[sp] = ([], [])
            leaf_name_dict[sp][0].append(i)

        for i in s2_leaf_list:
            sp = map_dict[i]
            if sp not in leaf_name_dict:
                leaf_name_dict[sp] = ([], [])
            leaf_name_dict[sp][1].append(i)

        # tandom repeat
        tandom_flag = False
        for sp in leaf_name_dict:
            s1_g_list = leaf_name_dict[sp][0]
            s2_g_list = leaf_name_dict[sp][1]

            for g1 in s1_g_list:
                for g2 in s2_g_list:
                    if if_tandom_function(g1, g2):
                        tandom_flag = True
                        break

        if tandom_flag:
            tandom_c_id.append(d_c_id)

    # move tandom to speciation
    for i in tandom_c_id:
        DS_dict['S'].append(i)
        DS_dict['D'].remove(i)

    dup_pair_dict = {}
    for c_id in DS_dict['D']:
        s1 = gene_tree_node_dict[c_id].clades[0]
        s2 = gene_tree_node_dict[c_id].clades[1]
        s1_list = [(map_dict[i.name], i.name) for i in s1.get_terminals()]
        s2_list = [(map_dict[i.name], i.name) for i in s2.get_terminals()]
        dup_pair_dict[c_id] = (s1_list, s2_list)

    all_gene_dict = {}
    for c_id in DS_dict['D'] + DS_dict['S']:
        all_gene_dict[c_id] = [(map_dict[i.name], i.name) for i in gene_tree_node_dict[c_id].get_terminals()]
        
    return DS_dict, dup_pair_dict, all_gene_dict


def check_duplication_for_all_tree_one_node(OGs, gene_tree_dir, species_tree, test_node, if_tandom_function):
    args_list = []
    args_id_list = []
    for og_id in OGs.OG_dict:
        # og_id = 'OG0001125'
        og = OGs.OG_dict[og_id]
        og.tree_file = "%s/%s_tree.txt" % (gene_tree_dir, og_id)

        # filter not good OG, species number more than 3, single species not more than 50 gene, top two species not have more than 90% gene
        # species more than 3
        if len([i for i in og.species_stat if og.species_stat[i] > 0]) < 3:
            continue
        if max(og.species_stat.values()) > 50:
            continue
        if sum(sorted(og.species_stat.values())[-2:]) / sum(og.species_stat.values()) > 0.9:
            continue
        if sum(og.species_stat.values()) <= 3:
            continue

        # do
        gene_tree = Phylo.read(og.tree_file, 'newick')
        gene_tree = add_clade_name(gene_tree)

        gene_list = [i.name for i in gene_tree.get_terminals()]
        map_dict = {}
        for i in gene_list:
            species, j = i.split("_")
            map_dict[i] = species

        args_list.append((gene_tree, species_tree, map_dict,
                          test_node, if_tandom_function))
        args_id_list.append(og_id)

    mlt_out = multiprocess_running(
        check_duplication_for_one_tree_one_node, args_list, 56, None, True, args_id_list)

    DS_dict = {}
    dup_pair_dict = {}
    all_gene_dict = {}

    for i in mlt_out:
        DS_dict[i] = mlt_out[i]['output'][0]
        dup_pair_dict[i] = mlt_out[i]['output'][1]
        all_gene_dict[i] = mlt_out[i]['output'][2]

    # # stat
    # keep_OG = 0
    # sum_OG = 0
    # for i in DS_dict:
    #     keep_OG += len(DS_dict[i]['D'])
    #     sum_OG += (len(DS_dict[i]['D']) + len(DS_dict[i]['S']))

    return DS_dict, dup_pair_dict, all_gene_dict


def get_gene_pair_Ks(gene_pairs_list):
    args_list = []
    args_id_list = []
    for i in gene_pairs_list:
        args_list.append((i,"/tmp"))
        args_id_list.append(i)

    mlt_out = multiprocess_running(
        quick_get_dNdS, args_list, 160, None, True, args_id_list)

    output_dict = {}
    for i in mlt_out:
        if mlt_out[i]['output'] == {}:
            continue
        for j in mlt_out[i]['output']:
            for k in mlt_out[i]['output'][j]:
                output_dict[i] = mlt_out[i]['output'][j][k][2]

    return output_dict


if __name__ == '__main__':
    from api.xuyuxing.comp_genome.gene_duplication_count import check_duplication_for_all_tree_one_node
    from lib.common.evolution.orthotools2 import OrthoGroups, read_species_info
    from lib.common.evolution.tree_operate import draw_ascii, add_clade_name, lookup_by_names
    from Bio import Phylo
    import re

    OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder/pt_file/OrthoFinder/Results_Jun03/Orthogroups/Orthogroups.tsv"
    species_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder/pt_file/OrthoFinder/Results_Jun03/Species_Tree/SpeciesTree_rooted.txt"
    gene_tree_dir = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder/pt_file/OrthoFinder/Results_Jun03/Resolved_Gene_Trees"
    ref_xlsx = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/Gel_ref.xlsx'

    # species tree
    species_tree = Phylo.read(species_tree_file, 'newick')
    species_tree = add_clade_name(species_tree)
    species_tree_node_dict = lookup_by_names(species_tree)
    draw_ascii(species_tree, clade_name=True)
    sp_list = [i.name for i in species_tree.get_terminals()]
    sp_info_dict = read_species_info(ref_xlsx, ['interpro_function_anno'])
    sp_info_dict = {i: sp_info_dict[i] for i in sp_info_dict if i in sp_list}

    # read fasta
    from lib.common.genome.seq_base import read_fasta_by_faidx
    for sp_id in sp_info_dict:
        sp = sp_info_dict[sp_id]
        sp.cds_dict = read_fasta_by_faidx(sp.cds_file)
        sp.pt_dict = read_fasta_by_faidx(sp.pt_file)

    # get function
    from api.xuyuxing.file_parser.interproscan_results import interproscan_results_parser
    from lib.common.fileIO import tsv_file_dict_parse
    import pickle

    ath_ortho_dir = '/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder_ath/pt_file/OrthoFinder/Results_Jun10/Orthologues/Orthologues_Ath'
    ath_anno_pyb = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function/all_annotation.pyb'

    TEMP = open(ath_anno_pyb, 'rb')
    data_dict = pickle.load(TEMP)
    TEMP.close()

    ath_go_dict = {}
    for i in data_dict:
        ath_go_dict[i] = data_dict[i][13].split("; ")
    
    for sp_id in sp_info_dict:
        # print(sp_id)
        sp = sp_info_dict[sp_id]

        # use ath go firstly
        ortho_file = ath_ortho_dir + "/Ath__v__%s.tsv" % sp_id
        ortho_file_info = tsv_file_dict_parse(ortho_file)

        to_ath_dict = {}
        for i in ortho_file_info:
            for j in ortho_file_info[i][sp_id].split(", "):
                to_ath_dict[j] = ortho_file_info[i]['Ath'].split(", ")

        to_ath_go_dict = {}
        for i in to_ath_dict:
            go_list = []
            for j in to_ath_dict[i]:
                go_list.extend(ath_go_dict[j])
            to_ath_go_dict[i] = list(set(go_list))

        sp.go_dict = to_ath_go_dict

        # use interpro
        if sp.interpro_function_anno:
            interproscan_output_tsv = sp.interpro_function_anno
            interpro_dict = interproscan_results_parser(interproscan_output_tsv)
            
            for i in interpro_dict:
                go_anno = []
                for j in interpro_dict[i].domain_list:
                    if not j["GO"] is None and not j['GO'] == '':
                        go_anno.extend(j["GO"].split("|"))
                go_anno = list(set(go_anno))

                if i not in sp.go_dict or len(sp.go_dict[i]) == 0:
                    sp.go_dict[i] = go_anno                

    # orthogroups
    OGs = OrthoGroups(OG_tsv_file=OG_tsv_file, species_list=sp_list)

    # get tandom function
    from lib.common.genome.genome_feature2 import read_gff_file
    from lib.xuyuxing.base.base_function import multiprocess_running
    from lib.common.genome.genome_feature2 import Gene

    def get_rank_dict(gff_file):
        gff_dict = read_gff_file(gff_file)
        gf_dict = gff_dict['gene']
        chr_dict = {}
        for gf_id in gf_dict:
            gf = gf_dict[gf_id]
            if gf.chr_id not in chr_dict:
                chr_dict[gf.chr_id] = []
            chr_dict[gf.chr_id].append(gf)
        rank_dict = {}
        for chr_id in chr_dict:
            num = 0
            for gf in sorted(chr_dict[chr_id], key=lambda x: x.start):
                num += 1
                rank_dict[gf.id] = (chr_id, num)
        return rank_dict

    def get_gff_info():
        ref_xlsx = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/Gel_ref.xlsx'
        sp_info_dict = read_species_info(ref_xlsx)

        args_list = []
        args_id_list = []

        for sp_id in sp_info_dict:
            sp = sp_info_dict[sp_id]
            args_list.append((sp.gff_file, ))
            args_id_list.append(sp_id)

        mlt_out = multiprocess_running(
            get_rank_dict, args_list, 160, None, True, args_id_list)

        all_rank_dict = {}
        for i in mlt_out:
            all_rank_dict[i] = mlt_out[i]['output']

        return all_rank_dict

    all_rank_dict = get_gff_info()

    def if_tandom(g1, g2):
        sp1, g1 = g1.split("_")
        sp2, g2 = g2.split("_")

        ch1, r1 = all_rank_dict[sp1][g1]
        ch2, r2 = all_rank_dict[sp2][g2]

        if sp1 == sp2 and ch1 == ch2 and abs(r1 - r2):
            return True
        else:
            return False

    # for test_node in species_tree_node_dict:
    #     print(test_node)
    #     check_duplication_for_all_tree_one_node(
    #         OGs, gene_tree_dir, species_tree, test_node, if_tandom)

    # for test_node in species_tree_node_dict:
    #     if test_node == 'N0':
    #         continue

    #     print(test_node)

    # running Ks
    # all_Ks_gene_pair_dict = {}
    # for test_node in species_tree_node_dict:
    #     if test_node == 'N0':
    #         continue

    #     print(test_node)

    #     DS_dict, dup_pair_dict, all_gene_dict = check_duplication_for_all_tree_one_node(
    #         OGs, gene_tree_dir, species_tree, test_node, if_tandom)

    #     # get gene pair seq
    #     dup_pair_seq = {}
    #     for og_id in dup_pair_dict:
    #         for c_id in dup_pair_dict[og_id]:
    #             s1_g_list = dup_pair_dict[og_id][c_id][0]
    #             s2_g_list = dup_pair_dict[og_id][c_id][1]

    #             for i in s1_g_list:
    #                 sp_id, gene_id = i
    #                 gene_id = gene_id.split("_")[1]
    #                 gi = Gene(gene_id, sp_id, model_aa_seq=sp_info_dict[sp_id].pt_dict[gene_id].seq,
    #                           model_cds_seq=sp_info_dict[sp_id].cds_dict[gene_id].seq)

    #                 for j in s2_g_list:
    #                     sp_id, gene_id = j
    #                     gene_id = gene_id.split("_")[1]
    #                     gj = Gene(
    #                         gene_id, sp_id, model_aa_seq=sp_info_dict[sp_id].pt_dict[gene_id].seq, model_cds_seq=sp_info_dict[sp_id].cds_dict[gene_id].seq)

    #                     dup_pair_seq[(gi, gj)] = (og_id, c_id)

    #     # get Ks for all pair

    #     Ks_dict = get_gene_pair_Ks(list(dup_pair_seq.keys()))

    #     all_Ks_gene_pair_dict[test_node] = dup_pair_seq, Ks_dict

    # import pickle

    # pyb_file = '/lustre/home/xuyuxing/Work/Gel/orcWGD/all_Ks_gene_pair_dict.pyb'
    # OUT = open(pyb_file, 'wb')
    # pickle.dump(all_Ks_gene_pair_dict, OUT)
    # OUT.close()

    # #
    import pickle

    pyb_file = '/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/WGD/all_Ks_gene_pair_dict.pyb'
    TEMP = open(pyb_file, 'rb')
    all_Ks_gene_pair_dict = pickle.load(TEMP)
    TEMP.close()

    # retend gene
    def test_a_node(test_node, ks_judge_function):
        import numpy as np

        DS_dict, dup_pair_dict, all_gene_dict = check_duplication_for_all_tree_one_node(
            OGs, gene_tree_dir, species_tree, test_node, if_tandom)

        dup_pair_seq, Ks_dict = all_Ks_gene_pair_dict[test_node]

        node_Ks_dict = {}

        for i in dup_pair_seq:
            if dup_pair_seq[i] not in node_Ks_dict:
                node_Ks_dict[dup_pair_seq[i]] = []
            if (i[0], i[1]) in Ks_dict:
                node_Ks_dict[dup_pair_seq[i]].append(Ks_dict[(i[0], i[1])])
            elif (i[1], i[0]) in Ks_dict:
                node_Ks_dict[dup_pair_seq[i]].append(Ks_dict[(i[1], i[0])])

        # filter by ks
        D_node_list = []
        S_node_list = []
        D_node_list_raw = []
        S_node_list_raw = []
        for og_id in DS_dict:
            S_node_list_raw.extend([(og_id, i) for i in DS_dict[og_id]['S']])
            D_node_list_raw.extend([(og_id, i) for i in DS_dict[og_id]['D']])

            S_node_list.extend([(og_id, i) for i in DS_dict[og_id]['S']])

            filter_out_D_node_id = []
            for d_node in DS_dict[og_id]['D']:
                ks_list = node_Ks_dict[(og_id, d_node)]
                if len(ks_list) > 0:
                    ks = np.median(ks_list)
                    # if (0.0 < ks < 1.0):
                    if ks_judge_function(ks):                    
                        D_node_list.append((og_id, d_node))
                    else:
                        S_node_list.append((og_id, d_node))
                else:
                    S_node_list.append((og_id, d_node))

        print(len(D_node_list), len(D_node_list + S_node_list), len(D_node_list)/len(D_node_list + S_node_list))

        # GO
        ## prepare gene list
        fore_node_dict = {}
        for og_id, node_id in D_node_list:
            fore_node_dict[(og_id, node_id)] = all_gene_dict[og_id][node_id]

        back_node_dict = {}
        for og_id, node_id in D_node_list + S_node_list:
            back_node_dict[(og_id, node_id)] = all_gene_dict[og_id][node_id]

        ## map function
        fore_node_go_dict = {}
        for og_id, node_id in fore_node_dict:
            node_function = []
            for sp_id, g_id in fore_node_dict[(og_id, node_id)]:
                g_id = g_id.split("_")[1]
                if g_id in sp_info_dict[sp_id].go_dict:
                    node_function.extend(sp_info_dict[sp_id].go_dict[g_id])
            fore_node_go_dict[(og_id, node_id)] = list(set(node_function))

        back_node_go_dict = {}
        for og_id, node_id in back_node_dict:
            node_function = []
            for sp_id, g_id in back_node_dict[(og_id, node_id)]:
                g_id = g_id.split("_")[1]
                if g_id in sp_info_dict[sp_id].go_dict:
                    node_function.extend(sp_info_dict[sp_id].go_dict[g_id])
            back_node_go_dict[(og_id, node_id)] = list(set(node_function))

        ## GO enrich
        from api.xuyuxing.resource.GO import  GO_enrichment_by_topGO
        fore_nodename_list = [i[0] + "_" + i[1] for i in fore_node_go_dict]
        back_nodename_dict = {i[0] + "_" + i[1]:back_node_go_dict[i] for i in back_node_go_dict}

        GO_dict = GO_enrichment_by_topGO(fore_nodename_list, back_nodename_dict, Ontology='BP')

        import pandas as pd

        df = pd.DataFrame(columns=(['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']))

        for i in GO_dict:
            df.loc[i] = [GO_dict[i][j] for j in ['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']]

        writer=pd.ExcelWriter('/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/WGD/'+test_node+".xlsx")
        df.to_excel(writer)
        writer.save()

        return fore_nodename_list, back_nodename_dict


    def ks_judge_function_N6(ks):
        return 0.86317289 - 0.2735924 * 1.5 < ks < 0.86317289 + 0.2735924 * 1.5

    def ks_judge_function_Vpl(ks):
        return 8.13054074e-01 - 3.30130199e-01 * 1.5 < ks < 8.13054074e-01 + 3.30130199e-01 * 1.5

    def ks_judge_function_Ash(ks):
        return 7.28064268e-01 - 2.61349661e-01 * 1.5 < ks < 7.28064268e-01 + 2.61349661e-01 * 1.5

    def ks_judge_function_N4(ks):
        return 1.09801254e+00 - 3.49635332e-01 * 1.5 < ks < 1.09801254e+00 + 3.49635332e-01 * 1.5

    def ks_judge_function_Aof(ks):
        return 0.2 < ks < 2

    def ks_judge_function_N9(ks):
        return 6.76080024e-01 - 3.29107510e-01 * 1.5 < ks < 6.76080024e-01 + 3.29107510e-01 * 1.5

    def ks_judge_function_Acom(ks):
        return 0.2 < ks < 8.39765692e-01 + 6.89888870e-01 * 1.5

    def ks_judge_function_Sly(ks):
        return 6.53568229e-01 - 2.18996450e-01 * 1.5 < ks < 6.53568229e-01 + 2.18996450e-01 * 1.5

    test_node_dict = {
        'N6': {
            'id':'N6',
            'function':ks_judge_function_N6,
        },
        'Vpl': {
            'id':'Vpl',
            'function':ks_judge_function_Vpl,
        },
        'Ash': {
            'id':'Ash',
            'function':ks_judge_function_Ash,
        },
        'N4': {
            'id':'N4',
            'function':ks_judge_function_N4,
        },
        'Aof': {
            'id':'Aof',
            'function':ks_judge_function_Aof,
        },
        'N6': {
            'id':'N6',
            'function':ks_judge_function_N6,
        },
        'N9': {
            'id':'N9',
            'function':ks_judge_function_N9,
        },
        'Acom': {
            'id':'Acom',
            'function':ks_judge_function_Acom,
        },
        'Sly': {
            'id':'Sly',
            'function':ks_judge_function_Sly,
        }
    }

    for test_node in test_node_dict:
        fore_nodename_list, back_nodename_dict = test_a_node(test_node, test_node_dict[test_node]['function'])
        test_node_dict[test_node]['output'] = (fore_nodename_list, back_nodename_dict)

    ## for OG level
    back_OG_dict = {}
    for test_node in test_node_dict:
        fore_nodename_list, back_nodename_dict = test_node_dict[test_node]['output']
        for i in back_nodename_dict:
            og_id, node_id = i.split("_", 1)
            if not og_id in back_OG_dict:
                back_OG_dict[og_id] = []
            back_OG_dict[og_id].extend(back_nodename_dict[i])

    for i in back_OG_dict:
        back_OG_dict[i] = list(set(back_OG_dict[i]))

    from api.xuyuxing.resource.GO import  GO_enrichment_by_topGO
    import pandas as pd

    fore_OG_dict = {}
    for test_node in test_node_dict:
        fore_nodename_list, back_nodename_dict = test_node_dict[test_node]['output']
        fore_OG_dict[test_node] = list(set([i.split("_", 1)[0] for i in fore_nodename_list]))
        print(test_node, len(fore_OG_dict[test_node]))

        GO_dict = GO_enrichment_by_topGO(fore_OG_dict[test_node], back_OG_dict, Ontology='BP')

        df = pd.DataFrame(columns=(['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']))

        for i in GO_dict:
            df.loc[i] = [GO_dict[i][j] for j in ['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']]

        writer=pd.ExcelWriter('/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/WGD/'+test_node+".OG_level.xlsx")
        df.to_excel(writer)
        writer.save()

    # core Kp gene
    sum_OG_list = []
    for test_node in ['Sly', 'Acom', 'Aof', 'N9']:
        sum_OG_list.extend(fore_OG_dict[test_node])

    from collections import Counter

    sum_OG_counter = Counter(sum_OG_list)
    core_Kp_OG = [i for i in sum_OG_counter if sum_OG_counter[i] >=3]

    GO_dict = GO_enrichment_by_topGO(core_Kp_OG, back_OG_dict, Ontology='BP')

    df = pd.DataFrame(columns=(['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']))

    for i in GO_dict:
        df.loc[i] = [GO_dict[i][j] for j in ['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']]

    writer=pd.ExcelWriter('/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/WGD/coreKP.OG_level.xlsx')
    df.to_excel(writer)
    writer.save()
    
    # venn
    import api.xuyuxing.plot.VennDiagram as venn

    for test_node in fore_OG_dict:
        labels, set_collections = venn.get_labels([fore_OG_dict[test_node], list(set(core_Kp_OG))], fill=['number'],
                                            return_set_collections=True)
                
        print(test_node, labels, int(labels['11'])/(int(labels['11'])+int(labels['10'])))


    for test_node in fore_OG_dict:
        labels, set_collections = venn.get_labels([list(set(fore_OG_dict[test_node] + fore_OG_dict['N4'])), list(set(core_Kp_OG))], fill=['number'],
                                            return_set_collections=True)
        
        print(test_node, labels, int(labels['11'])/(int(labels['11'])+int(labels['10'])))

    # N46
    only_N46_list = list(set(fore_OG_dict['N6'] + fore_OG_dict['N4']) - set(core_Kp_OG))

    GO_dict = GO_enrichment_by_topGO(only_N46_list, back_OG_dict, Ontology='BP')

    df = pd.DataFrame(columns=(['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']))

    for i in GO_dict:
        df.loc[i] = [GO_dict[i][j] for j in ['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']]

    writer=pd.ExcelWriter('/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/WGD/only_N46.OG_level.xlsx')
    df.to_excel(writer)
    writer.save()

    # N46
    N46_list = list(set(fore_OG_dict['N6'] + fore_OG_dict['N4']))

    GO_dict = GO_enrichment_by_topGO(N46_list, back_OG_dict, Ontology='BP')

    df = pd.DataFrame(columns=(['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']))

    for i in GO_dict:
        df.loc[i] = [GO_dict[i][j] for j in ['GO_id', 'Rank_in_weight', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank_in_classic', 'classic', 'KS', 'weight']]

    writer=pd.ExcelWriter('/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/WGD/N46.OG_level.xlsx')
    df.to_excel(writer)
    writer.save()


    # -----------------
    # get speciation Ks
    work_dir = '/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/speciation'
    species_tree_file = "/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder_old/pt_file/OrthoFinder/Results_Jun01/Species_Tree/SpeciesTree_rooted.txt"
    orthogroups_for_species_tree = '/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder_old/pt_file/OrthoFinder/Results_Jun01/Species_Tree/Orthogroups_for_concatenated_alignment.txt'
    OG_tsv_file = '/lustre/home/xuyuxing/Work/Gel/orcWGD/orthofinder_old/pt_file/OrthoFinder/Results_Jun01/Orthogroups/Orthogroups.tsv'
    ref_xlsx = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/Gel_ref.xlsx'

    from lib.common.evolution.orthotools2 import OrthoGroups, read_species_info
    from lib.common.fileIO import read_list_file
    from lib.common.genome.genome_feature2 import Gene
    from lib.common.genome.seq_base import read_fasta_by_faidx
    from Bio import Phylo
    from lib.common.evolution.tree_operate import draw_ascii, add_clade_name, lookup_by_names

    # read species
    species_tree = Phylo.read(species_tree_file, 'newick')
    species_tree = add_clade_name(species_tree)
    species_tree_node_dict = lookup_by_names(species_tree)
    draw_ascii(species_tree, clade_name=True)
    sp_list = [i.name for i in species_tree.get_terminals()]
    sp_info_dict = read_species_info(ref_xlsx)
    sp_info_dict = {i: sp_info_dict[i] for i in sp_info_dict if i in sp_list}

    # read fasta
    for sp_id in sp_info_dict:
        sp = sp_info_dict[sp_id]
        sp.cds_dict = read_fasta_by_faidx(sp.cds_file)
        sp.pt_dict = read_fasta_by_faidx(sp.pt_file)

    # read OGs
    OGs = OrthoGroups('for_speciation', OG_tsv_file, species_list=sp_list)
    used_GO_list = read_list_file(orthogroups_for_species_tree)

    # running Ks
    spec_pair_seq = {}
    for test_node_id in species_tree_node_dict:
        test_node = species_tree_node_dict[test_node_id]
        if test_node.is_terminal():
            continue
        s1 = test_node.clades[0]
        s2 = test_node.clades[1]
        
        sp1 = [i.name for i in s1.get_terminals()]
        sp2 = [i.name for i in s2.get_terminals()]

        print(sp1, sp2)

        # get gene pair seq
        for og_id in used_GO_list:
            og = OGs.OG_dict[og_id]
            sp1_gene_list = []
            for sp_id in sp1:
                for gene in og.gene_dict[sp_id]:
                    gene.sp_id = sp_id
                    gene.model_aa_seq = sp_info_dict[sp_id].pt_dict[gene.id].seq
                    gene.model_cds_seq = sp_info_dict[sp_id].cds_dict[gene.id].seq
                    sp1_gene_list.append(gene)

            sp2_gene_list = []
            for sp_id in sp2:
                for gene in og.gene_dict[sp_id]:
                    gene.sp_id = sp_id
                    gene.model_aa_seq = sp_info_dict[sp_id].pt_dict[gene.id].seq
                    gene.model_cds_seq = sp_info_dict[sp_id].cds_dict[gene.id].seq
                    sp2_gene_list.append(gene)


            for i in sp1_gene_list:
                for j in sp2_gene_list:
                    if i == j:
                        continue
                    spec_pair_seq[(i, j)] = (og_id, test_node_id)
    
    Ks_dict = get_gene_pair_Ks(list(spec_pair_seq.keys()))


    


    import pickle

    pyb_file = '/lustre/home/xuyuxing/Work/Gel/orcWGD/Ks/speciation/all_Ks_gene_pair_dict.pyb'
    OUT = open(pyb_file, 'wb')
    pickle.dump((spec_pair_seq, Ks_dict), OUT)
    OUT.close()





import uuid
import pandas as pd
from itertools import combinations
import networkx as nx
from interlap import InterLap
from collections import Counter
from lib.common.genome.seq_base import read_fasta_by_faidx
import lib.common.sqlite_command as sc
from lib.common.os import mkdir, multiprocess_running
from lib.common.genome.genome_feature2 import read_gff_file, get_gf_db_meta_dict, gf_info_retrieval, write_gff_file
from lib.xuyuxing.math.set_operating import interval_length_sum
from lib.common.math.interval import section, merge_intervals, interval_minus_set, overlap_between_interval_set, \
    group_by_intervals_with_overlap_threshold
from api.xuyuxing.comp_genome.orthofinder import OG_tsv_file_parse
from lib.xuyuxing.math.stats import get_threshold

# about weight score
def get_ortho_support(OG_tsv_file):

    OG_dict = OG_tsv_file_parse(OG_tsv_file)
    seq_support_dict = {}
    for OG_id in OG_dict:
        empty_num = len([i for i in OG_dict[OG_id] if len(OG_dict[OG_id][i]) == 1 and OG_dict[OG_id][i][0] == ''])
        total_num = len(OG_dict[OG_id])
        support_num = total_num - empty_num

        for i in OG_dict[OG_id]:
            for j in OG_dict[OG_id][i]:
                if j == '':
                    continue
                seq_support_dict[j] = (support_num, total_num)

    return seq_support_dict


# def get_weight(idt_len, query_length, OG_support_num):
#     if query_length > 200:
#         length_score = 5
#     elif query_length > 150:
#         length_score = 4
#     elif query_length > 100:
#         length_score = 3
#     elif query_length > 50:
#         length_score = 2
    
#     if OG_support_num <= 2:
#         support_score = 0.2
#     else:
#         support_score = 1

#     # return (idt_len/aln_len) * (aln_len/query_length) * length_score
#     return (idt_len/query_length) * length_score * support_score

# def get_weight(pst_ratio, q_cover, q_length, OG_support_num):
#     if pst_ratio > 0.7:
#         pst_score = 10
#     elif pst_ratio > 0.6:
#         pst_score = 8
#     elif pst_ratio > 0.5:
#         pst_score = 6
#     elif pst_ratio > 0.4:
#         pst_score = 4
#     else:
#         pst_score = 1

#     if q_cover > 0.7:
#         q_cover_score = 10
#     elif q_cover > 0.6:
#         q_cover_score = 8
#     elif q_cover > 0.5:
#         q_cover_score = 6
#     elif q_cover > 0.4:
#         q_cover_score = 4
#     else:
#         q_cover_score = 1

#     if q_length > 200:
#         length_score = 10
#     elif q_length > 150:
#         length_score = 8
#     elif q_length > 100:
#         length_score = 6
#     elif q_length > 50:
#         length_score = 4
#     else:
#         length_score = 1

#     if OG_support_num > 5:
#         OG_score = 10
#     elif OG_support_num > 4:
#         OG_score = 8
#     elif OG_support_num > 3:
#         OG_score = 6
#     elif OG_support_num > 2:
#         OG_score = 4
#     else:
#         OG_score = 1

#     # return (idt_len/aln_len) * (aln_len/query_length) * length_score
#     # return (idt_len * (OG_support_num + 1))
#     return pst_score * q_cover_score * length_score * OG_score

# def get_ev_weight(gf):
#     q_cover = gf.evidence_indicator['query_coverage']
#     idt_ratio = gf.evidence_indicator['identity']

#     mRNA = gf.sub_features[0]
#     query_length = int(mRNA.qualifiers['Target_Length'][0])

#     OG_support_num = gf.OG_support[0]

#     ev_weight_value = get_weight(idt_ratio, q_cover, query_length, OG_support_num)

#     return ev_weight_value

# weight_dict = {i:get_ev_weight(evidence_dict[i]) for i in evidence_dict}

# with open("1.txt" , 'w') as f:
#     for i in weight_dict:
#         f.write(str(weight_dict[i])+"\n")

# def get_weight(idt_len, query_length, OG_support_num):

#     # return (idt_len/aln_len) * (aln_len/query_length) * length_score
#     # return (idt_len * (OG_support_num + 1))
#     return (idt_len * (OG_support_num + 1))/query_length


# def get_ev_weight(gf):

#     mRNA = gf.sub_features[0]
#     idt_len = mRNA.qualifiers['idt_len']
#     query_length = int(mRNA.qualifiers['Target_Length'][0])
#     OG_support_num = gf.OG_support[0]
#     ev_weight_value = get_weight(idt_len, query_length, OG_support_num)

#     return ev_weight_value

# weight_dict = {i:get_ev_weight(evidence_dict[i]) for i in evidence_dict}

# with open("2.txt" , 'w') as f:
#     for i in weight_dict:
#         f.write(str(weight_dict[i])+"\n")

def get_weight(idt_len, query_length, OG_support_num):

    # return (idt_len/aln_len) * (aln_len/query_length) * length_score
    # return (idt_len * (OG_support_num + 1))
    return (idt_len * (OG_support_num + 1))


def get_ev_weight(gf):

    mRNA = gf.sub_features[0]
    idt_len = mRNA.qualifiers['idt_len']
    query_length = int(mRNA.qualifiers['Target_Length'][0])
    OG_support_num = gf.OG_support[0]
    ev_weight_value = get_weight(idt_len, query_length, OG_support_num)

    return ev_weight_value

# weight_dict = {i:get_ev_weight(evidence_dict[i]) for i in evidence_dict}

# with open("3.txt" , 'w') as f:
#     for i in weight_dict:
#         f.write(str(weight_dict[i])+"\n")


def mark_ev_weight(evidence_dict):
    output_ev_dict = {}
    for ev_id in evidence_dict:
        ev_gf = evidence_dict[ev_id]
        ev_weight_value = get_ev_weight(ev_gf)
        ev_gf.weight = ev_weight_value
        output_ev_dict[ev_id] = ev_gf
    return output_ev_dict

def make_cds_vector(evidence_dict, vector_range=(1,1)):
    cds_weight_interlap = InterLap()

    coordinate_list = [vector_range[0], vector_range[1]]
    for ev_id in evidence_dict:
        ev_gf = evidence_dict[ev_id]
        ev_weight_value = ev_gf.weight

        mRNA = ev_gf.sub_features[0]
        for cds in mRNA.sub_features:
            cds_weight_interlap.add((cds.start, cds.end, ev_weight_value))
            coordinate_list.append(cds.start)
            coordinate_list.append(cds.end)

    coordinate_list = sorted(list(set(coordinate_list)))

    cds_vector_interlap = InterLap()
    getted_range = {}
    for i in range(0, len(coordinate_list)-1):
        start = coordinate_list[i]
        end = coordinate_list[i+1]

        if (start, start) not in getted_range:
            w_s_total = sum(
                [w for s, e, w in cds_weight_interlap.find((start, start))])
            getted_range[(start, start)] = w_s_total
            cds_vector_interlap.add((start, start, w_s_total))

        if (end, end) not in getted_range:
            w_e_total = sum(
                [w for s, e, w in cds_weight_interlap.find((end, end))])
            getted_range[(end, end)] = w_e_total
            cds_vector_interlap.add((end, end, w_e_total))

        if end - start > 2:
            interval = (start + 1, end - 1)
            if interval not in getted_range:
                w_i_total = sum(
                    [w for s, e, w in cds_weight_interlap.find(interval)])
                getted_range[interval] = w_i_total
                cds_vector_interlap.add((interval[0], interval[1], w_i_total))

    return cds_vector_interlap


def get_vector_score(vector_interlap, start, end):

    w = [w for s, e, w in vector_interlap.find((start, end))]
    if len(w) > 1:
        raise ValueError("cds error: %d, %d" % (start, end))
    elif len(w) == 0:
        w = 0.0
    else:
        w = w[0]

    return w


# def get_ev_gf_score(gf, cds_vector_interlap):

#     ev_weight_value = get_ev_weight(gf)

#     mRNA = gf.sub_features[0]
#     query_length = int(mRNA.qualifiers['Target_Length'][0])

#     sum_w = 0
#     length = 0
#     for cds in mRNA.sub_features:
#         for i in range(cds.start, cds.end+1):
#             length += 1
#             sum_w += get_vector_score(cds_vector_interlap, i, i)

#     gf_score = (sum_w / query_length / 3, ev_weight_value)

#     return gf_score

# def get_ev_gf_score(gf, cds_vector_interlap):

#     ev_weight_value = get_ev_weight(gf)

#     mRNA = gf.sub_features[0]
#     query_length = int(mRNA.qualifiers['Target_Length'][0])

#     sum_w = 0
#     length = 0
#     for cds in mRNA.sub_features:
#         for i in range(cds.start, cds.end+1):
#             length += 1
#             sum_w += get_vector_score(cds_vector_interlap, i, i)

#     gf_score = (sum_w / query_length, ev_weight_value)
#     # gf_score = (sum_w, ev_weight_value)

#     return gf_score

def get_supp_weight_score(gf, cds_vector_interlap):

    mRNA = gf.sub_features[0]
    query_length = int(mRNA.qualifiers['Target_Length'][0])

    sum_w = 0
    length = 0
    for cds in mRNA.sub_features:
        for s,e,w in cds_vector_interlap.find((cds.start, cds.end)):
            if_flag, delta = section((s,e),(cds.start, cds.end),True)
            sum_w = sum_w + (max(delta) - min(delta) + 1) * w
            length += (max(delta) - min(delta) + 1)

    support_weight = sum_w
    # gf_score = (sum_w, ev_weight_value)

    return support_weight, length

def mark_ev_supp_weight(evidence_dict, cds_vector_interlap):
    output_ev_dict = {}
    for ev_id in evidence_dict:
        ev_gf = evidence_dict[ev_id]
        ev_weight_value, cds_aln_length = get_supp_weight_score(ev_gf, cds_vector_interlap)
        ev_gf.supp_weight = ev_weight_value
        ev_gf.cds_aln_length = cds_aln_length
        output_ev_dict[ev_id] = ev_gf
    return output_ev_dict

# evidence_dict = mark_ev_supp_weight(evidence_dict, cds_vector_interlap)
# supp_weight_list = [evidence_dict[i].supp_weight for i in evidence_dict]
# with open("5.txt" , 'w') as f:
#     for i in supp_weight_list:
#         f.write(str(i)+"\n")


# import math
# log_supp_weight_list = [math.log2(evidence_dict[i].supp_weight) for i in evidence_dict]

# with open("6.txt" , 'w') as f:
#     for i in log_supp_weight_list:
#         f.write(str(i)+"\n")

# def get_ev_gf_score(gf, cds_vector_interlap):

#     ev_weight_value = get_ev_weight(gf)

#     mRNA = gf.sub_features[0]
#     query_length = int(mRNA.qualifiers['Target_Length'][0])

#     sum_w = 0
#     length = 0
#     for cds in mRNA.sub_features:
#         for s,e,w in cds_vector_interlap.find((cds.start, cds.end)):
#             if_flag, delta = section((s,e),(cds.start, cds.end),True)
#             sum_w = sum_w + (max(delta) - min(delta) + 1) * w

#     gf_score = (sum_w / query_length, ev_weight_value)
#     # gf_score = (sum_w, ev_weight_value)

#     return gf_score

# def mark_ev_score_for_gf_dict(input_gf_dict, cds_vector_interlap):
#     output_gf_dict = {}
#     args_list = []
#     args_id_list = []
#     for i in input_gf_dict:
#         gf = input_gf_dict[i]
#         args_list.append((gf, cds_vector_interlap))
#         args_id_list.append(i)

#     gf_score_out = multiprocess_running(get_ev_gf_score, args_list, 56, args_id_list=args_id_list)

#     gf_score_dict = {}
#     for i in gf_score_out:
#         gf_score_dict[i] = list(gf_score_out[i]['output'])

#     # get rank score
#     support_rank = sorted(list(gf_score_dict.keys()), key=lambda x:gf_score_dict[x][0], reverse=True)
#     self_rank = sorted(list(gf_score_dict.keys()), key=lambda x:gf_score_dict[x][1], reverse=True)

#     for i in gf_score_dict:
#         gf_score_dict[i].append((support_rank.index(i) + 1) * (self_rank.index(i) + 1))


#     output_gf_dict = {}
#     for i in gf_score_dict:
#         gf = input_gf_dict[i]
#         gf.gf_score_detail = gf_score_dict[i]
#         gf.gf_score = gf_score_dict[i][2]
#         output_gf_dict[i] = gf

#     return output_gf_dict

# def mark_ev_score_for_gf_dict(input_gf_dict, cds_vector_interlap):
#     gf_score_dict = {}
#     for i in input_gf_dict:
#         gf = input_gf_dict[i]
#         gf_score_dict[i] = list(get_ev_gf_score(gf, cds_vector_interlap))

#     # get rank score
#     support_rank = sorted(list(gf_score_dict.keys()), key=lambda x:gf_score_dict[x][0], reverse=True)
#     self_rank = sorted(list(gf_score_dict.keys()), key=lambda x:gf_score_dict[x][1], reverse=True)

#     for i in gf_score_dict:
#         gf_score_dict[i].append((support_rank.index(i) + 1) * (self_rank.index(i) + 1))


#     output_gf_dict = {}
#     for i in gf_score_dict:
#         gf = input_gf_dict[i]
#         gf.gf_score_detail = gf_score_dict[i]
#         gf.gf_score = gf_score_dict[i][2]
#         output_gf_dict[i] = gf

#     return output_gf_dict

def mark_ev_score_for_gf_dict(input_gf_dict):
    gf_score_dict = {}
    for i in input_gf_dict:
        gf = input_gf_dict[i]
        gf_score_dict[i] = [gf.supp_weight, gf.weight]

    # get rank score
    support_rank = sorted(list(gf_score_dict.keys()), key=lambda x:gf_score_dict[x][0], reverse=True)
    self_rank = sorted(list(gf_score_dict.keys()), key=lambda x:gf_score_dict[x][1], reverse=True)

    for i in gf_score_dict:
        gf_score_dict[i].append((support_rank.index(i) + 1) * (self_rank.index(i) + 1))


    output_gf_dict = {}
    for i in gf_score_dict:
        gf = input_gf_dict[i]
        gf.gf_score_detail = gf_score_dict[i]
        gf.gf_score = gf_score_dict[i][2]
        output_gf_dict[i] = gf

    return output_gf_dict

def mark_OG_support_for_gf_dict(input_gf_dict, seq_support_dict):
    output_gf_dict = {}

    for i in input_gf_dict:
        gf = input_gf_dict[i]

        q_id = gf.id.split("_")[0]
        if q_id in seq_support_dict:
            gf.OG_support = seq_support_dict[q_id]
        else:
            gf.OG_support = (0,0)

        output_gf_dict[i] = gf
    
    return output_gf_dict

# about exclusivity
def get_overlap_ratio(gf1, gf2, similarity_type):
    gf1_cds_interval = [(j.start, j.end)
                        for j in gf1.sub_features[0].sub_features]
    gf2_cds_interval = [(j.start, j.end)
                        for j in gf2.sub_features[0].sub_features]

    overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
        gf1_cds_interval, gf2_cds_interval, similarity_type=similarity_type)

    return overlap_ratio


def mark_exclusivity_for_gf_dict(input_gf_dict):
    ev_id_list = list(input_gf_dict.keys())

    score_pair_dict = {}
    for i, j in combinations(ev_id_list, 2):
        score_pair_dict[(i, j)] = get_overlap_ratio(
            input_gf_dict[i], input_gf_dict[j], 'shorter_overlap_coverage')
    
    output_gf_dict = {}
    for ev_id_i in ev_id_list:
        gf = input_gf_dict[ev_id_i]
        exclusivity_list = []
        for ev_id_j in ev_id_list:
            if ev_id_i == ev_id_j:
                continue
            if (ev_id_i, ev_id_j) in score_pair_dict:
                overlap_ratio = score_pair_dict[(ev_id_i, ev_id_j)]
            else:
                overlap_ratio = score_pair_dict[(ev_id_j, ev_id_i)]

            if overlap_ratio > 0.1:
                exclusivity_list.append(ev_id_j)
        gf.exclusivity = exclusivity_list
        gf.exclusivity.append(ev_id_i)
        output_gf_dict[ev_id_i] = gf

    return output_gf_dict


# about evidence get and cluster
def load_map_files(reference_genome_table, WPGmapper_dir):
    # read ref genome taxon_id
    ref_db = pd.read_excel(reference_genome_table)
    ref_taxon_id_list = []
    for index in ref_db.index:
        taxon_id = str(ref_db.loc[index]['id'])
        if not pd.isna(taxon_id):
            ref_taxon_id_list.append(taxon_id)

    # load map database
    ref_WPGmapper_dict = {}
    for taxon_id in ref_taxon_id_list:
        ref_WPGmapper_dict[taxon_id] = {}
        ref_WPGmapper_dict[taxon_id]['genblasta'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genblasta.db"
        ref_WPGmapper_dict[taxon_id]['genewise'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genewise.db"
        ref_WPGmapper_dict[taxon_id]['pep_db'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genewise.pep.db"
        ref_WPGmapper_dict[taxon_id]['cds_db'] = WPGmapper_dir + \
            "/WPGmapper/" + taxon_id + "/genewise.cds.db"

    return ref_WPGmapper_dict


def min_cds_start(cds_range_list):
    return min([min(i) for i in cds_range_list])


def get_overlap_ratio(gf1,gf2,similarity_type):
    gf1_cds_interval = [(j.start, j.end) for j in gf1.sub_features[0].sub_features]
    gf2_cds_interval = [(j.start, j.end) for j in gf2.sub_features[0].sub_features]

    overlap_ratio, overlap_length, overlap = overlap_between_interval_set(gf1_cds_interval, gf2_cds_interval, similarity_type=similarity_type)
    
    return overlap_ratio


def higher_in_lower_group(lower_merged_group, higher, cds_interval_dict, similarity_type, threshold):
    grouped_flag = False

    for lower in lower_merged_group:
        jaccord_score = overlap_between_interval_set(
            cds_interval_dict[lower], cds_interval_dict[higher], similarity_type)[0]
        if jaccord_score >= threshold:
            grouped_flag = True
            break

    return grouped_flag


def merge_evidence_by_cds_intervals_identity(ev_id_list, cds_interval_dict, similarity_type, threshold):

    sorted_by_lower_bound = sorted(
        ev_id_list, key=lambda x: min_cds_start(cds_interval_dict[x]))
    merged = Counter()

    num = 0
    for higher in sorted_by_lower_bound:
        if len(merged) == 0:
            merged[num] = [higher]
            num += 1
        else:
            g_id_list = list(merged.keys())
            g_id_list.reverse()

            grouped_flag = False
            for g_id in g_id_list:
                grouped_flag = higher_in_lower_group(
                    merged[g_id], higher, cds_interval_dict, similarity_type, threshold)
                if grouped_flag == True:
                    merged[g_id].append(higher)
                    break

            if not grouped_flag:
                merged[num] = [higher]
                num += 1

    return merged


def evidence_cluster(evidence_dict, similarity_type='jaccord_score', threshold=0.3):
    if len(evidence_dict) > 0:

        # CDS interval list
        cds_interval_dict = {}
        gene_interval_dict = {}
        for i in evidence_dict:
            # if not evidence_dict[i].map_quality_pass:
            #     continue

            gene_interval_dict[i] = (
                evidence_dict[i].start, evidence_dict[i].end)
            cds_interval_dict[i] = [
                (j.start, j.end) for j in evidence_dict[i].sub_features[0].sub_features]

        big_group_dict = group_by_intervals_with_overlap_threshold(
            gene_interval_dict, 0)

        # # load evidences into interlap
        # evidence_interlap = InterLap()
        # evidence_interlap.update(
        #     [(gene_interval_dict[i][0], gene_interval_dict[i][1], i) for i in gene_interval_dict])

        # merge intervals into cluster
        ev_cluster_dict = {}
        num = 0
        for bg_id in big_group_dict:
            cds_interval_dict_in_group = {
                ev_id: cds_interval_dict[ev_id] for ev_id in big_group_dict[bg_id]['list']}

            ev_cluster_dict_in_group = merge_evidence_by_cds_intervals_identity(
                list(cds_interval_dict_in_group.keys()), cds_interval_dict_in_group, similarity_type, threshold)

            for i in ev_cluster_dict_in_group:
                num += 1
                ev_cluster_dict[num] = ev_cluster_dict_in_group[i]

        # ev_cluster_range_dict = {}
        # for cl_id in ev_cluster_dict:
        #     ev_cluster_range_dict[cl_id] = merge_intervals([gene_interval_dict[i] for i in ev_cluster_dict[cl_id]])[0]

        return ev_cluster_dict

    else:
        return Counter()


def extract_evidence(contig, strand, ref_WPGmapper_dict, given_range=None):

    # get all useable evidence on the contig(strand)

    evidence_dict = {}

    for taxon_id in ref_WPGmapper_dict:
        # print(taxon_id)

        gws_db = ref_WPGmapper_dict[taxon_id]['genewise']
        meta_dict = get_gf_db_meta_dict(gws_db)

        # read mRNA info

        for table_name in meta_dict['gene'][0]:
            # print(table_name)
            if given_range is None:
                sql_cmd_string = 'SELECT * FROM %s WHERE contig_name = \"%s\" AND strand = \"%s\"' % (
                    table_name, contig, strand)
            else:
                sql_cmd_string = 'SELECT * FROM %s WHERE contig_name = \"%s\" AND strand = \"%s\" AND start <= %d AND end >= %d' % (
                    table_name, contig, strand, max(given_range), min(given_range))

            data_list = sc.sqlite_execute(sql_cmd_string, gws_db)
            for gf_db_info_tuple in data_list:
                id_tmp, gf_name, type_tmp, contig_name, start, end, strand, daughter, qualifiers = gf_db_info_tuple
                gf = gf_info_retrieval(gf_db_info_tuple, 0, 'A', gws_db)

                mRNA_gf = gf.sub_features[0]
                qualifiers = mRNA_gf.qualifiers

                # query coverage
                coverage = min((int(qualifiers['Target_End'][0]) - int(
                    qualifiers['Target_Start'][0]) + 1) / int(qualifiers['Target_Length'][0]), 1.00)

                # aln aa
                aln_aa_len = int(
                    qualifiers['Target_End'][0]) - int(qualifiers['Target_Start'][0]) + 1

                # identity
                identity = int(qualifiers['idt_len']) / \
                    int(qualifiers['aln_len'])

                gf.evidence_indicator = {
                    "query_coverage": coverage,
                    "aln_aa_len": aln_aa_len,
                    "identity": identity,
                }

                gf.db_path = "%s.%s.%s" % (taxon_id, table_name, id_tmp)

                gf.score  = max(0.1, float(gf.sub_features[0].qualifiers['score'][0]))

                evidence_dict[gf.id] = gf

    return evidence_dict


def mark_map_quality_for_gf_dict(input_gf_dict, args):
    for ev_id in input_gf_dict:
        # quality control
        coverage = input_gf_dict[ev_id].evidence_indicator["query_coverage"]
        aln_aa_len = input_gf_dict[ev_id].evidence_indicator["aln_aa_len"]
        identity = input_gf_dict[ev_id].evidence_indicator["identity"]
        score = input_gf_dict[ev_id].score

        if coverage >= args.min_cover and aln_aa_len >= args.min_aa_len and identity >= args.min_identity and score >= args.min_score:
            input_gf_dict[ev_id].map_quality_pass = True
        else:
            input_gf_dict[ev_id].map_quality_pass = False

    return input_gf_dict


# about graph
def no_red_by_jaccord(ev_in_cluster_dict):
    no_red_ev_dict_clst = evidence_cluster(ev_in_cluster_dict, similarity_type='jaccord_score', threshold=0.9)
    no_red_ev_dict = {}
    for i in no_red_ev_dict_clst:
        gf_id = sorted([j for j in no_red_ev_dict_clst[i]], key=lambda x:ev_in_cluster_dict[x].gf_score, reverse=True)[0]
        no_red_ev_dict[gf_id] = ev_in_cluster_dict[gf_id]
    return no_red_ev_dict


def load_a_node(graph, give_node_id, remaining_selectable_ev_list, evidence_dict_range):
    # print(give_node_id)
    # print(remaining_selectable_ev_list)
    uuid_id, seq_id = give_node_id

    for i in remaining_selectable_ev_list:
        uuid_id = uuid.uuid1().hex
        node_id = (uuid_id, i)

        if seq_id == 'START':
            print(node_id)

        graph.add_edge(give_node_id, node_id, weight=evidence_dict_range[i].gf_score)

        remaining_selectable_ev_list_for_i = list(set(remaining_selectable_ev_list) - set(evidence_dict_range[i].exclusivity))
        if len(remaining_selectable_ev_list_for_i) == 0:
            graph.add_edge(node_id, 'END', weight=1)
            # print(node_id)
        else:
            load_a_node(graph, node_id, remaining_selectable_ev_list_for_i, evidence_dict_range)


def use_graph_to_get_rep(evidence_dict_range):
    
    G = nx.DiGraph()
    uuid_id = uuid.uuid1().hex
    start = (uuid_id, 'START')

    load_a_node(G, start, list(evidence_dict_range.keys()), evidence_dict_range)
    # paths = nx.johnson(G, weight="weight")
    # paths[start]["END"]

    return nx.shortest_path(G,start,"END",weight='weight')

# Brute-force
def brute_force_get_rep(ev_in_cluster_dict):
    gf_score_rank = sorted(list(ev_in_cluster_dict.keys()), key=lambda x:ev_in_cluster_dict[x].gf_score)

#todo

    return ev_in_cluster_dict


# greedy algorithm
def greedy_get_rep(ev_in_cluster_dict):
    gf_score_rank = sorted(list(ev_in_cluster_dict.keys()), key=lambda x:ev_in_cluster_dict[x].gf_score)

    rep_gf_list = []
    ex_gf_set = set()
    for i in gf_score_rank:
        if i not in ex_gf_set:
            rep_gf_list.append(i)
            ex_gf_set = ex_gf_set | set(ev_in_cluster_dict[i].exclusivity)

    for i in ev_in_cluster_dict:
        if i in rep_gf_list:
            ev_in_cluster_dict[i].is_rep = True
        else:
            ev_in_cluster_dict[i].is_rep = False

    return ev_in_cluster_dict


# def rep_gf_judge(evidence_dict_cleaned):
#     # give me cleaned evidence_dict

#     evidence_cluster_dict = evidence_cluster(evidence_dict_cleaned, similarity_type='shorter_overlap_coverage', threshold=0.1)
    
#     rep_gf_dict = {}
#     output_dict = {}
#     for cl_id in evidence_cluster_dict:
#         print(cl_id)
#         ev_id_list = evidence_cluster_dict[cl_id]
#         if len(ev_id_list) == 1:

#             gf_id = ev_id_list[0]
#             gf = evidence_dict_cleaned[gf_id]
#             gf.exclusivity = [gf_id]
#             gf.gf_score = 1
#             gf.gf_score_detail = [0,get_ev_weight(gf),1]
#             gf.is_rep = True

#             output_dict[gf_id] = gf
#         else:
#             ev_in_cluster_dict = {ev_id:evidence_dict_cleaned[ev_id] for ev_id in evidence_cluster_dict[cl_id]}
#             cds_vector_interlap = make_cds_vector(ev_in_cluster_dict)
#             ev_in_cluster_dict = mark_ev_score_for_gf_dict(ev_in_cluster_dict, cds_vector_interlap)
#             ev_in_cluster_dict = mark_exclusivity_for_gf_dict(ev_in_cluster_dict)
#             ev_in_cluster_dict = greedy_get_rep(ev_in_cluster_dict)

#             for i in ev_in_cluster_dict:
#                 output_dict[i] = ev_in_cluster_dict[i]
    
#     return output_dict

def rep_gf_cluster(ev_in_cluster_dict):

    ev_in_cluster_dict = mark_ev_score_for_gf_dict(ev_in_cluster_dict)
    ev_in_cluster_dict = mark_exclusivity_for_gf_dict(ev_in_cluster_dict)
    ev_in_cluster_dict = greedy_get_rep(ev_in_cluster_dict)

    return ev_in_cluster_dict

def rep_gf_judge(evidence_dict_cleaned):
    # give me cleaned evidence_dict

    evidence_cluster_dict = evidence_cluster(evidence_dict_cleaned, similarity_type='shorter_overlap_coverage', threshold=0.1)
    
    args_list = []
    for cl_id in evidence_cluster_dict:
        
        ev_in_cluster_dict = {ev_id:evidence_dict_cleaned[ev_id] for ev_id in evidence_cluster_dict[cl_id]}

        args_list.append((ev_in_cluster_dict, ))

    rep_gf_cluster_out = multiprocess_running(rep_gf_cluster, args_list, 56 , silence=True)

    output_dict = {}
    for i in rep_gf_cluster_out:
        ev_in_cluster_dict = rep_gf_cluster_out[i]['output']
        for i in ev_in_cluster_dict:
            output_dict[i] = ev_in_cluster_dict[i]
    
    return output_dict


#########
 

def pseudogene_main(args):
    mkdir(args.output_dir, True)

    contig_list = list(read_fasta_by_faidx(args.target_genome_fasta).keys())

    # load WPGmapper data
    ref_WPGmapper_dict = load_map_files(
        args.reference_genome_table, args.WPGmapper_dir)

    seq_support_dict = get_ortho_support(args.OG_tsv_file)

    # parse a contig
    args_list = []
    for contig in contig_list:
        args_list.append((contig, ref_WPGmapper_dict, seq_support_dict, args))

    multiprocess_running(pseudogene_by_contig, args_list,
                         args.threads, silence=True)


def pseudogene_by_contig(contig, strand, ref_WPGmapper_dict, seq_support_dict, args):

    evidence_dict = extract_evidence(contig, strand, ref_WPGmapper_dict)
    evidence_dict = mark_map_quality_for_gf_dict(evidence_dict, args)

    evidence_dict_clean = {i:evidence_dict[i] for i in evidence_dict if evidence_dict[i].map_quality_pass}

    evidence_dict_clean = mark_OG_support_for_gf_dict(evidence_dict_clean, seq_support_dict)
    evidence_dict_clean = mark_ev_weight(evidence_dict_clean)

    threshold, mu, sigma, statistic, pvalue = get_threshold([evidence_dict_clean[i].weight for i in evidence_dict_clean])

    evidence_dict_clean_weight = {i:evidence_dict[i] for i in evidence_dict if evidence_dict[i].weight >= threshold}
    cds_vector_interlap = make_cds_vector(evidence_dict_clean_weight)
    evidence_dict_clean_weight = mark_ev_supp_weight(evidence_dict_clean_weight, cds_vector_interlap)


    evidence_dict_clean_weight = rep_gf_judge(evidence_dict_clean_weight)

    return evidence_dict_clean_weight


weight_list = [evidence_dict_clean[i].weight for i in evidence_dict_clean]

with open("weight_list.txt" , 'w') as f:
    for i in weight_list:
        f.write(str(i)+"\n")

score_list = [evidence_dict_clean[i].score * evidence_dict_clean[i].OG_support[0] for i in evidence_dict_clean]

score_list = [evidence_dict_clean[i].score for i in evidence_dict_clean if evidence_dict_clean[i].score < 200]

with open("score_list.txt" , 'w') as f:
    for i in score_list:
        f.write(str(i)+"\n")

score_list = []
for i in evidence_dict_clean:
    plus_score = 1

    if evidence_dict_clean[i].OG_support[0] >= 2:
        plus_score = 10
    
    if evidence_dict_clean[i].cluster_support / evidence_dict_clean[i].score > 2:
        plus_score = 10
    
    score = evidence_dict_clean[i].score * plus_score
    score_list.append(math.log2(score))


with open("score_list.txt" , 'w') as f:
    for i in score_list:
        f.write(str(i)+"\n")

score_del_list = []
for i in evidence_dict_clean:
    plus_score = 1

    if evidence_dict_clean[i].OG_support[0] >= 2:
        plus_score = 10
    
    if evidence_dict_clean[i].cluster_support / evidence_dict_clean[i].score > 2:
        plus_score = 10

    if plus_score == 10:
        score = evidence_dict_clean[i].score
        score_del_list.append(math.log2(score))


with open("score_del_list.txt" , 'w') as f:
    for i in score_del_list:
        f.write(str(i)+"\n")


rep_gf_list = [evidence_dict_cleaned_mark_rep[gf_id] for gf_id in evidence_dict_cleaned_mark_rep if evidence_dict_cleaned_mark_rep[gf_id].is_rep is True]
write_gff_file(rep_gf_list, "/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pseudogene/PC/T267555N0C000/T267555N0C000.WPGmaper.new.skip_all/rep2.gff3")

import pickle

file_name = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pseudogene/PC/T267555N0C000/T267555N0C000.WPGmaper.new.skip_all/2.tmp.py"

data_list = [(i.gf_score_detail[0],i.gf_score_detail[1]) for i in rep_gf_list]

data_list = [(evidence_dict[i].weight,evidence_dict[i].supp_weight) for i in evidence_dict]
data_list = [output_dict[i].cluster_support + output_dict[i].score for i in output_dict]

OUT = open(file_name, 'wb')
pickle.dump(data_list, OUT)
OUT.close()


TEMP = open(file_name, 'rb')
data_list = pickle.load(TEMP)
TEMP.close()

TEMP = open(file_name, 'rb')
evidence_dict = pickle.load(TEMP)
TEMP.close()

# score
score_list = [float(evidence_dict[i].sub_features[0].qualifiers['score'][0]) for i in evidence_dict]
q_len_list = [float(evidence_dict[i].sub_features[0].qualifiers['Target_Length'][0]) for i in evidence_dict]
idt_list = [float(evidence_dict[i].sub_features[0].qualifiers['idt_len']) for i in evidence_dict]
idt_ratio_list = [float(evidence_dict[i].sub_features[0].qualifiers['idt_len'])/float(evidence_dict[i].sub_features[0].qualifiers['Target_Length'][0]) for i in evidence_dict]
score_OG_list = [float(evidence_dict[i].sub_features[0].qualifiers['score'][0]) * evidence_dict[i].OG_support[0] for i in evidence_dict]

score_list = [float(evidence_dict[i].sub_features[0].qualifiers['score'][0]) for i in evidence_dict]


for i in evidence_dict:
    evidence_dict[i].score  = max(0.1, float(evidence_dict[i].sub_features[0].qualifiers['score'][0]))

def get_cluster_support(evidence_dict):
    output_dict = {}
    evidence_cluster_dict = evidence_cluster(evidence_dict, similarity_type='jaccord_score', threshold=0.5)

    for i in evidence_cluster_dict:
        print(i)
        # num = 0
        for j in evidence_cluster_dict[i]:
            # num+=1
            # print(num)
            gf_j = evidence_dict[j]
            cluster_support = 0
            for k in evidence_cluster_dict[i]:
                if j == k:
                    continue
                gf_k = evidence_dict[k]
                overlap_ratio = get_overlap_ratio(gf_j,gf_k,"jaccord_score")
                if overlap_ratio > 0.8:
                    cluster_support += gf_k.score
            gf_j.cluster_support = cluster_support
            output_dict[j] = gf_j
    
    return output_dict
                




import math
import matplotlib.pyplot as plt
import numpy as np

# x = [math.log10(i[0]+1) for i in data_list]
# y = [math.log10(i[1]+1) for i in data_list]

# x = [i[0] for i in data_list]
# y = [i[1] for i in data_list]

x = idt_list
y = score_list

# 由pyplot创建figure，并由axes等对象的方法进行绘图
fig, ax = plt.subplots(figsize=(10,10))

#绘图
ax.scatter(x, y)

#展示
plt.show()


import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable

x = [math.log2(i[0]+1) for i in data_list]
y = [math.log2(i[1]+1) for i in data_list]

fig, ax = plt.subplots(figsize=(15, 15))

# the scatter plot:
ax.scatter(x, y)

# Set aspect of the main axes.
ax.set_aspect(1.)

# create new axes on the right and on the top of the current axes
divider = make_axes_locatable(ax)
# below height and pad are in inches
ax_histx = divider.append_axes("top", 1.2, pad=0.1, sharex=ax)
ax_histy = divider.append_axes("right", 1.2, pad=0.1, sharey=ax)

# make some labels invisible
ax_histx.xaxis.set_tick_params(labelbottom=False)
ax_histy.yaxis.set_tick_params(labelleft=False)

# now determine nice limits by hand:
binwidth = 0.25
xymax = max(np.max(np.abs(x)), np.max(np.abs(y)))
xymin = max(np.max(np.abs(x)), np.max(np.abs(y)))
lim = (int(xymax/binwidth) + 1)*binwidth

bins = np.arange(0, lim + binwidth, binwidth)
ax_histx.hist(x, bins=bins)
ax_histy.hist(y, bins=bins, orientation='horizontal')

# the xaxis of ax_histx and yaxis of ax_histy are shared with ax,
# thus there is no need to manually adjust the xlim and ylim of these
# axis.

ax_histx.set_yticks([0, 50, 100])
ax_histy.set_xticks([0, 50, 100])

# ax.set_xlim(3, 10)
# ax.set_ylim(1, 5)

plt.show()


if __name__ == '__main__':
    import argparse

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='DeepTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for CleanFastq
    parser_a = subparsers.add_parser('CleanFastq',
                                     help='clean fastq files\n',
                                     description='')

    parser_a.add_argument('raw_fastq1', type=str, help='raw fastq file 1')
    parser_a.add_argument('raw_fastq2', type=str, help='raw fastq file 2')
    parser_a.add_argument('tag', type=str, help='tag for fastq')
    parser_a.add_argument("-a", "--adapter", help="clean adapter ('illumina' or 'mgi_seq' or sequence)",
                          type=str, default='illumina')

    # argparse for BamSplit
    parser_a = subparsers.add_parser('BamSplit',
                                     help='split a big read sorted bam file by reads number\n',
                                     description='')

    parser_a.add_argument('input_bam_file', type=str, help='raw bam file')
    parser_a.add_argument("-r", "--reads_per_file", help="how many reads should in one bam file (default:1000000)",
                          type=int, default=1000000)
    parser_a.add_argument('-c', "--by_contig",
                          help='split bam by contig', action='store_true')
    parser_a.add_argument('-l', "--log_file", type=str,
                          help='path for log file (default:None)', default=None)
    parser_a.add_argument('-k', "--keep_head",
                          help='keep raw head', action='store_true')

    # argparse for MappedReadsComparing
    parser_a = subparsers.add_parser('MappedReadsComparing',
                                     help='comparing the mapped reads CIgar from two reference genome \n',
                                     description='')

    parser_a.add_argument('fastq_file', type=str, help='the clean reads file')
    parser_a.add_argument('mapped_bam_file_1', type=str, help='the bam file 1 from reference genome A')
    parser_a.add_argument('mapped_bam_file_2', type=str, help='the bam file 2 from reference genome B')
    parser_a.add_argument('reads_id_file', type=str, help='the reads id list from fastq file')
    parser_a.add_argument('tag1', type=str, help='A flag')
    parser_a.add_argument('tag2', type=str, help='B flag')
    parser_a.add_argument('tag_file', type=str, help='the output tag file')


    # argparse for ReadMapStats
    parser_a = subparsers.add_parser('ReadMapStats',
                                     help='stats the reads mapping hits for a bam file\n',
                                     description='')

    parser_a.add_argument('input_bam_file', type=str, help='raw bam file')
    parser_a.add_argument('-l', "--log_file", type=str,
                          help='path for log file (default:None)', default=None)
    parser_a.add_argument(
        "-t", "--threads", help="cpu num, (default:20)", default=20, type=int)
    parser_a.add_argument("-r", "--reads_per_file", help="how many reads should in one bam file (default:1000000)",
                          type=int, default=1000000)

    # argparse for Fastq2pbbam
    parser_a = subparsers.add_parser('Fastq2pbbam',
                                     help="""
                                     use subread fastq data to maker a pacbio bam file to cheat ccs program in pacbio github
                                     
                                     $ dnapi.py 20190627.Gelata-red1_iso.cell1.subreads.fastq
                                     GTATCAACGCAGAGTAC
                                     
                                     $ cutadapt -g GTATCAACGCAGAGTAC -e 0.3 --action lowercase -o left.cut.fq 20190627.Gelata-red1_iso.cell1.subreads.fastq
                                     $ cutadapt -a GTACTCTGCGTTGATAC -e 0.3 --action lowercase -o right.cut.fq 20190627.Gelata-red1_iso.cell1.subreads.fastq
                                     """,
                                     description="""
                                     use subread fastq data to maker a pacbio bam file to cheat ccs program in pacbio github\n
                                     
                                     $ dnapi.py 20190627.Gelata-red1_iso.cell1.subreads.fastq\n
                                     GTATCAACGCAGAGTAC\n
                                     
                                     $ cutadapt -g GTATCAACGCAGAGTAC -e 0.3 --action lowercase -o left.cut.fq 20190627.Gelata-red1_iso.cell1.subreads.fastq\n
                                     $ cutadapt -a GTACTCTGCGTTGATAC -e 0.3 --action lowercase -o right.cut.fq 20190627.Gelata-red1_iso.cell1.subreads.fastq\n
                                     """)

    parser_a.add_argument('fastq_file', type=str,
                          help='input fastq file, maybe from ncbi')
    parser_a.add_argument('leaf_cut_fastq', type=str, help='leaf cutadapt')
    parser_a.add_argument('right_cut_fastq', type=str, help='right cutadapt')
    parser_a.add_argument('bam_file', type=str, help='output bam file')

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # BamSplit
    if args_dict["subcommand_name"] == "BamSplit":
        from src.xuyuxing.tools.deeptools import BamSplit_main
        BamSplit_main(args)

    # ReadMapStats
    elif args_dict["subcommand_name"] == "ReadMapStats":
        """
        class abc(object):
            pass

        args = abc()

        args.log_file = '/lustre/home/xuyuxing/Work/Other/saif/Meth/test/log'
        args.input_bam_file = '/lustre/home/xuyuxing/Work/Other/saif/Meth/test/test.bam'
        args.reads_per_file = 10000
        args.threads = 50
        """
        from src.xuyuxing.tools.deeptools import ReadMapStats_main
        ReadMapStats_main(args)

    # Fastq2pbbam
    elif args_dict["subcommand_name"] == "Fastq2pbbam":
        """
        class abc():
            pass

        args = abc()
        args.fastq_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/trans/full_length/bam_file/20190627.Gelata-red1_iso.cell1.subreads.fastq'
        args.leaf_cut_fastq = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/trans/full_length/bam_file/left.cut.fq'
        args.right_cut_fastq = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/trans/full_length/bam_file/right.cut.fq'
        args.bam_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/trans/full_length/bam_file/new.subreads.bam'
        """
        from src.xuyuxing.tools.deeptools import Fastq2pbbam_main
        Fastq2pbbam_main(args)

    # CleanFastq
    elif args_dict["subcommand_name"] == "CleanFastq":
        from src.xuyuxing.tools.deeptools import clean_fastq_main
        clean_fastq_main(args.tag, args.raw_fastq1,
                         args.raw_fastq2, args.adapter)

    # MappedReadsComparing
    elif args_dict["subcommand_name"] == "MappedReadsComparing":
        from src.xuyuxing.tools.MappedReadsComparing import MappedReadsComparing_main
        MappedReadsComparing_main(args)




###

import ete3



def subtree(tree_file, leaf1, leaf2, root_leaf):
    t = ete3.Tree(tree_file)
    if not root_leaf is None:
        t.set_outgroup(root_leaf)
    node = t.get_common_ancestor(leaf1, leaf2)
    return node


def phyloxml_read_ete(phyloxml_file):
    project = ete3.Phyloxml()
    project.build_from_file(phyloxml_file)
    tree = project.get_phylogeny()[0]
    return tree


def phyloxml_write_ete(tree, phyloxml_file):
    project = ete3.Phyloxml()
    project.add_phylogeny(tree)
    with open(phyloxml_file,'w') as f:
        project.export(f)
    return phyloxml_file

#####




#
# """
# Jupyter
# t.render("%%inline")
# """
#
# from pyvirtualdisplay import Display
# from ete3 import Tree
# display = Display(visible=False, size=(1024, 768), color_depth=24)
# display.start()
#
# t = Tree("/lustre/home/xuyuxing/Work/Cau_HGT/tmp/phyout/phyout.phb")
#
# t = Tree('((((H,K)D,(F,I)G)B,E)A,((L,(N,Q)O)J,(P,S)M)C);', format=1)
#
#
# len(t)
#
# print(t.get_ascii(show_internal=True))
#
# #include root
# for i in t.traverse("postorder"):
#     print(i.name)
#
# #no root
# for i in t.iter_descendants("postorder"):
#     print(i.name)
#
# # Browse the tree from a specific leaf to the root
# node = t.search_nodes(name="H")[0]
# while node:
#    print(node)
#    node = node.up
#
# abs()
#
# t.set_outgroup(t&"H")
#
# display.stop()